/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.layout;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.Date;


public class About
{
   private static final String TITLE = "";

   // ---- Default Port & Version

   public static final int DEFAULT_PORT = 1234;

   public static final String VERSION = "1.4";
   public static final String SUBVERSION = "2021.01.28";  // !!! build !!!
   public static final String VERSION_COMPLETE = VERSION+" ("+SUBVERSION+")";
   public static final String COPYRIGHT = "Copyright (c) 2002-"+(new Date().getYear()+1900);
   public static final String HOVERBALL_WEBSITE = "www.hoverball.org";

   // ---- Paddings

   private static final int padx = 32;
   private static final int pady = 16;

   private static void drawStringRight(Graphics g, String s, int x, int y)
   {
      int w = g.getFontMetrics().stringWidth(s);
      g.drawString(s,x-w,y);
   }

   // ---- main()-Parameter-Unifier -------------------------------------------

   public static String arg (String[] args)
   {
      String arg = "";
      for (int i=0; i<args.length; ++i) arg += args[i];
      return arg;
   }

   // ---- Contents -----------------------------------------------------------

   private static class Contents extends JPanel
   {
      public final JButton button;

      public Contents ()
      {
         super(new BorderLayout());

         // ---- Top [Logo]

         JPanel a = new JPanel() {
            public void paint (Graphics g)
            {
               Dimension size = getSize();

               g.setColor(Color.white);
               g.fillRect(0,0,size.width,size.height);

               // ---- Logo

               int w = 326; //(int)(image.getWidth(this)*r);
               int h = 256; //(int)(image.getHeight(this)*r);

               g.drawImage(Layout.logo_png,-76+(size.width-w)/2,+10+(size.height-h)/2,w,h,null);

               // ---- Title & Version

               ((Graphics2D)g).setRenderingHint(
                     RenderingHints.KEY_TEXT_ANTIALIASING,
                     RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

               g.setColor(Color.black);
               g.setFont(Layout.fontHoverball);
               drawStringRight(g,"Hoverball",size.width-padx,size.height-pady-20);
               g.setFont(Layout.fontVersion);
               drawStringRight(g,"Version "+VERSION_COMPLETE,size.width-padx,size.height-pady);

               // ---- Line

               g.setColor(Layout.colorBackground.darker());
               g.drawLine(0,size.height-1,size.width,size.height-1);
            }
         };
         a.setPreferredSize(new Dimension(400,184));
         a.setBackground(Color.white);

         // ---- Body [Text & Button]

         JPanel c = new JPanel() {
            public void paintComponent (Graphics g)
            {
               super.paintComponent(g);

               Dimension size = getSize();

               // ---- Text

               ((Graphics2D)g).setRenderingHint(
                     RenderingHints.KEY_TEXT_ANTIALIASING,
                     RenderingHints.VALUE_TEXT_ANTIALIAS_ON);

               g.setFont(Layout.fontAbout);
               int y = g.getFontMetrics().getAscent();
               g.drawString(COPYRIGHT,padx,y+pady -1);
               g.drawString("Stefan Bornhofen, Matthias Bornhofen",padx,y+2*pady +0);
               g.drawString(HOVERBALL_WEBSITE,padx,size.height-pady-4 -1);
            }
         };

         JPanel b = new JPanel(new GridBagLayout());
         b.add(c,
               new Constraints(0,0,1,1,1.0,1.0,new Insets(0,0,0,0),Constraints.SOUTHWEST,Constraints.BOTH));
         b.add(button = new MyButton("OK",false,70),
               new Constraints(1,0,1,1,0.0,1.0,new Insets(0,0,pady,padx-8),Constraints.SOUTHEAST,Constraints.NONE));
         // ----

         add(a,"North");
         add(b,"Center");
      }
   }

   // ---- About Window -------------------------------------------------------

   public final JInternalFrame frame;

   public About (Component component)
   {
      Contents contents = new Contents();

      frame = new MyInternalFrame(TITLE,false,true,false,false);
      frame.setLayer(JLayeredPane.MODAL_LAYER);

      frame.setSize(new Dimension(400,312));  // will be centered by Controller

      final Component component_ = (component == null)? frame : component;
      contents.button.addActionListener(new ActionListener() {
         public void actionPerformed(ActionEvent e) { component_.setVisible(false); }
      });

      frame.setDefaultCloseOperation(JInternalFrame.HIDE_ON_CLOSE);

      Container j = frame.getContentPane();
      j.setLayout(new BorderLayout());

      j.add(contents,"Center");
   }
}
