/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.layout;

/*

Cursor Controls

ESC[H    moves cursor to home position (0, 0)
ESC[{line};{column}H
ESC[{line};{column}f    moves cursor to line #, column #
ESC[#A   moves cursor up # lines
ESC[#B   moves cursor down # lines
ESC[#C   moves cursor right # columns
ESC[#D   moves cursor left # columns
ESC[#E   moves cursor to beginning of next line, # lines down
ESC[#F   moves cursor to beginning of previous line, # lines down
ESC[#G   moves cursor to column #
ESC[#;#R    reports current cursor line and column
ESC[s    saves the current cursor position              -- not supported very often
ESC[u    restores the cursor to the last saved position -- not supported very often

Erase Functions

ESC[J    clears the screen
ESC[0J   clears from cursor until end of screen
ESC[1J   clears from cursor to beginning of screen
ESC[2J   clears entire screen
ESC[K    clears the current line
ESC[0K   clears from cursor to end of line
ESC[1K   clears from cursor to start of line
ESC[2K   clears entire line

8-16 Colors

Black    30    40
Red      31    41
Green    32    42
Yellow   33    43
Blue     34    44
Magenta  35    45
Cyan     36    46
White    37    47
Reset    0     0

Beispiel

# Set style to bold, red foreground.
\x1b[1;31mHello
# Set style to dimmed white foreground with red background.
\x1b[2;37;41mWorld

https://gist.github.com/fnky/458719343aabd01cfb17a3a4f7296797
http://ascii-table.com/ansi-escape-sequences.php

*/    
    
public class Ansi {
   public static boolean terminal = (System.console() != null && System.getenv().get("TERM") != null);
   public static final char ESC = 0x1B;
   
   public static String clear () { return ESC+"[2J"; }
   public static String home () { return ESC+"[H"; }
   public static String up (int n) { return ESC+"["+n+"A"; }
}
