package hoverball.net;

import java.io.*;

// No more IOException will be thrown.
// Instead receiver's catchIOException() will be invoked.

public class NetAgent implements Receiver
{
/*private!*/ public Socket socket = null;       // underlying Socket if real network (TCP)
	private Receiver receiver = null;   // Agent's receiver
	private boolean closed = false;     // only close once, please!
	private static final int TIMEOUT = 3000;  // Connection Timeout

    // ---- public methods of Receiver
	public void receive (Address sender, String message) {}
	public void catchIOException (IOException exception, String where)
	{
	   if (Socket.log!=null) Socket.log.println(""+socket+" exception ("+exception+") in "+where);
	}

    public NetAgent (Socket socket) // server side
    {
	  this(socket,null);
      new ReceiverThread(this).start();
    }

    public NetAgent (Address serverAddress) throws IOException // client side
    {
	   this(serverAddress,null);
	   // receiver thread not yet started
    }

    private NetAgent (Socket socket, Receiver receiver) // server side
    {
 	   this.receiver = (receiver == null)? this : receiver;
 	   this.socket = socket;
    }

    private NetAgent (Address serverAddress, Receiver receiver) throws IOException // client side
    {
 	   this.receiver = (receiver == null)? this : receiver;
if (Socket.log != null) Socket.log.println("SOCKET ERSTELLEN: "+serverAddress);
 	   this.socket = new Socket(serverAddress);  // wenn das scheitert, dann dauert dieser Befehl ca. 10 Sekunden!
    }

    public void send (Object data)
    {
    	try {
    		socket.send(new Message(socket.getLocalAddress(), socket.getRemoteAddress(), data.toString()));
    	} catch(IOException e) { receiver.catchIOException(e,"send"); }
    }

    public boolean connect (Address server, Object data)
    // client connection request
    // note: the receiver thread is not yet running
    {
      if (!server.isValid()) return false; // invalid address.
      try {
         socket.send(new Message(socket.getLocalAddress(),server,data.toString()));
         socket.setTimeout(TIMEOUT);
         Message reply = socket.receive();  // waiting for reply ...
         socket.setTimeout(0);
         receiver.receive(reply.sender,reply.data);
         new ReceiverThread(this).start();
         return true;
      } catch(IOException e) { receiver.catchIOException(e,"connect"); }
      return false;
    }

    public synchronized void close ()
    {
      if (closed) return;
      try {
    	  if (socket!=null) socket.close();
      } catch (IOException e) { receiver.catchIOException(e,"close"); }
      closed = true;
    }

   // ---- receiver thread

   static class ReceiverThread extends Thread
   {
      private NetAgent agent;

      ReceiverThread (NetAgent agent)
      {
//APPLET THREAD BUG//
    	 super("Hoverball Receiver Thread");
         //super(Socket.group,Socket.thread("Receiver",agent.socket.getLocalAddress()));
         this.setPriority(Thread.NORM_PRIORITY);  // ??
         this.agent = agent;
      }

      public void run ()
      {
         try {
        	Message message = null;
            while (!agent.closed) {
            	Thread.yield();
            	message = agent.socket.receive();
            	agent.receiver.receive(message.sender,message.data);
             }
         } catch (IOException e) { // (hoffentlich) nur SocketException: socket closed.
        	agent.receiver.catchIOException(e,"receive");
         } finally {
            agent.close();
         }
      }
   }
}
