/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.serial;


// (channel 3 2 "The Ultimates!" RoboBob 23456 3.2)

// ###:  name==null = (channel 3 2)

public class Channel extends Serial
{
   public int t;          // team number
   public int n;          // unit number in team
   public String team;    // team name
   public String name;    // unit name
   public int color;      // preferred color (0xRRGGBB)
   public String host;    // unit client's host

   public Object clone ()
   {
      return new Channel(t,n,clone(team),clone(name),color,clone(host)).next(clone(next));
   }

   public Channel (int t, int n) { this(t,n,null,null,0,null); }

   public Channel (int t, int n, String team, String name, int color, String host)
   {
      this.t = t;
      this.n = n;
      this.team = (team == null)? null : ""+team;
      this.name = (name == null)? null : ""+name;
      this.color = color;
      this.host = (host == null)? null : ""+host;
   }

   public void serialize (StringBuffer buffer)
   {
      buffer.append("(channel ");
      buffer.append(putInt(t));
      buffer.append(' ');
      buffer.append(putInt(n));
      if (name != null) {
         buffer.append(' ');
         buffer.append(putString(team));
         buffer.append(' ');
         buffer.append(putString(name));
         buffer.append(' ');
         buffer.append(putInt(color));
         buffer.append(' ');
         buffer.append(putString(host));
      }
      buffer.append(')');
   }

   public static Serial deserialize (Object[] objects)
   {
      String keyword = getString(objects[0]); if (!keyword.equals("channel")) return null;
      if (objects.length != 3 && objects.length != 7) throw new IllegalNumberOfArguments();

      int t = getInt(objects[1]);
      int n = getInt(objects[2]);
      if (objects.length == 3) return new Channel(t,n);

      String team = getString(objects[3]);
      String name = getString(objects[4]);
      int color = getInt(objects[5]);
      String host = getString(objects[6]);
      return new Channel(t,n,team,name,color,host);
   }
}

