/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.serial;

import hoverball.math.*;


// ---- Relative Disk Info
// (node 0 n x1 x2 x3)
// (ball t n x1 x2 x3)
// (unit t n x1 x2 x3 "...")

public class DiskRelative extends DiskInfo
{
   public int what;
   public int t;
   public int n;

   public Vector x;   // position (euler)

   public String message;


   public Object clone ()
   {
      return new DiskRelative(what,t,n,x,message);
   }

   public DiskRelative (int what, int t, int n, Vector x, String message)
   {
      this.what = what;
      this.t = t;
      this.n = n;

      this.x = (x==null)? null : new Vector(x);

      this.message = (message==null)? null : new String(message);
   }

   public void serialize (StringBuffer buffer)
   {
      buffer.append('(');
      buffer.append(putWhat(what));
      buffer.append(' ');
      buffer.append(putInt(t));
      buffer.append(' ');
      buffer.append(putInt(n));
      if (x != null) {
         buffer.append(' ');
         buffer.append(putDouble(x.x));   // pos.phi
         buffer.append(' ');
         buffer.append(putDouble(x.y));   // pos.r / 2*pi*R
         buffer.append(' ');
         buffer.append(putDouble(x.z));   // a
      }
      if (message != null) {
         buffer.append(' ');
         buffer.append(putString(message));
      }
      buffer.append(')');
   }

   public static Serial deserialize (Object[] objects)
   {
      String keyword = getString(objects[0]);
      int what = getWhat(keyword); if (what == UNKNOWN) return null;

      int t = getInt(objects[1]);
      int n = getInt(objects[2]);

      int i = 3;
      if (objects.length == 4 && what == UNIT) {       // (what t n message)
         String message = getString(objects[i++]);

         return new DiskRelative(what,t,n,null,message);
      }
      else if (objects.length == 6) {                  // (what t n x1 x2 x3)
         double x1 = getDouble(objects[i++]);
         double x2 = getDouble(objects[i++]);
         double x3 = getDouble(objects[i++]);
         Vector x = new Vector(x1,x2,x3);

         return new DiskRelative(what,t,n,x,null);
      }
      else if (objects.length == 7 && what == UNIT) {  // (what t n x1 x2 x3 message)
         double x1 = getDouble(objects[i++]);
         double x2 = getDouble(objects[i++]);
         double x3 = getDouble(objects[i++]);
         Vector x = new Vector(x1,x2,x3);
         String message = getString(objects[i++]);

         return new DiskRelative(what,t,n,x,message);
      }
      else
         throw new IllegalNumberOfArguments();
   }
}

