/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.simulator;

import hoverball.math.Complex;
import hoverball.math.Matrix;


// ---- class Disque ----------------------------------------------------------

class Disque
/*
   Une disque est l'unique objet qui peut etre placé dans le Terrain.
   En effet, il s'agit d'un palet qui glisse sur le terrain. Quand il
   touche un mur ou une autre disque, il rebondit.
*/
{
   public static final int INCONNU = hoverball.serial.DiskInfo.UNKNOWN;
   public static final int BALLON  = hoverball.serial.DiskInfo.BALL;
   public static final int UNITE   = hoverball.serial.DiskInfo.UNIT;

   // ---- Kernel Variablen

   double trame = 0;                           // Zeitraster für Auswertung von Aktion, Polarisation und Reibung
   double temps = Double.POSITIVE_INFINITY;    // Zeit seit letzter Kollision  (init: +inf)
   Disque collision = this;                    // Disque der letzten Kollision (init: this)
   double collision_Q = 0;                     // Polarisation der Disque der letzten Kollision

   // ---- Terrain Variablen

   final int quoi;  // quoi = unit/ball
   final int t;     // t = team-id
   final int n;     // n = disk-id in team t

   final double m;             // Masse
   final double r;             // Radius
   final double J;             // Trägheitsmoment

   double  Q = 0;              // Ladung
   Matrix  X = new Matrix();   // Position (als ONB)
   Complex p = new Complex();  // Impuls (2D)
   double  L = 0;              // Drehimpuls
   Complex F = new Complex();  // Translatorische Kraft
   double  T = 0;              // Drehmoment

   String message = null;      // Message

   Disque (Disque disque)
   {
      this(disque.quoi,disque.t,disque.n,disque.m,disque.r,disque.Q,
           disque.X,disque.p,disque.L,disque.message);
   }

   Disque (int quoi, int t, int n, double m, double r, double Q,
           Matrix X, Complex p, double L, String message)
   {
      this.quoi = quoi;
      this.t = t;
      this.n = n;

      this.m = m;
      this.r = r;
      this.J = 0.5*m*r*r;

      this.Q = Q;

      this.X = (X==null)? null : new Matrix(X);
      this.p = (p==null)? null : new Complex(p);
      this.L = L;

      this.message = (message==null)? null : new String(message);
   }

   // ---- Virtual Disque

   Complex xQ() { return new Complex(); }
}
