/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.simulator;

import hoverball.layout.*;

import java.awt.*;
import javax.swing.*;

import java.util.Vector;


class SimulatorFrame extends MyInternalFrame
{
   private JLabel labelChannels = null;
   private JLabel labelControls = null;
   private JLabel labelState = null;
   private JLabel labelTime = null;

   public double duration = 0;  // for "Complete"-Label

   SimulatorFrame (String address, Simulator to_close)
   {
      super(Layout.getTitle("Simulator"),false,false,false,false);

      this.to_close = to_close;  // dispose -> close! (must be directly)
      
      setSize(Layout.BROWSER_WIDTH,Layout.SIMULATOR_HEIGHT);

      Container j = getContentPane();
      j.setLayout(new GridBagLayout());

      int padx = Layout.padx;
      int tabx = Layout.tabx;
      int pady = 8;

      // ---- Labels

      JPanel banner = new JPanel() {
         public Dimension getPreferredSize() { return new Dimension(0,Layout.BANNER); }
         public void paintComponent(Graphics g) { super.paintComponent(g);
            Layout.drawBanner(g,getSize(),
               "Simulator",
               "Hoverball "+About.VERSION,false);
         }
      }; banner.setSize(banner.getPreferredSize());
      j.add(banner,
          new Constraints(0,0,2,1,1.0,0.0,new Insets(0,0,10,0)));

      JLabel label = null;
      j.add(new JLabel("Network Address:"),
          new Constraints(0,2,1,1,0.0,0.0,new Insets(0,padx+tabx,6,0)));
      j.add(label = new JLabel(address),
          new Constraints(1,2,1,1,0.0,0.0,new Insets(0,padx,6,0)));
      label.setFont(Layout.fontUser);

      j.add(new JLabel("Connected:"),
          new Constraints(0,3,1,1,0.0,0.0,new Insets(0,padx+tabx,0,0)));
      j.add(labelChannels = new JLabel(""),
          new Constraints(1,3,1,1,1.0,0.0,new Insets(0,padx,0,0)));
      j.add(labelControls = new JLabel(""),
          new Constraints(1,4,1,1,1.0,0.0,new Insets(0,padx,0,0)));
      labelChannels.setFont(Layout.fontUser);
      labelControls.setFont(Layout.fontUser);

      // ---- Vertical Filler

      j.add(new JLabel(),new Constraints(0,5,3,1,0.0,1.0,new Insets(0,0,0,0)));

      // ---- Status & Time

      JPanel panel = new JPanel(new GridBagLayout());
      panel.setOpaque(false);

      panel.add(labelState = new JLabel(),
          new Constraints(0,0,1,1,1.0,0.0,new Insets(0,0,pady,0)));
      panel.add(labelTime = new JLabel(),
          new Constraints(1,0,1,1,1.0,0.0,new Insets(0,0,pady,0)));

      labelState.setBackground(getBackground().darker());
      labelState.setForeground(Layout.colorForeground);
      labelState.setOpaque(true);
      labelState.setFont(Layout.fontBold);
      labelState.setHorizontalAlignment(JLabel.RIGHT);
      labelState.setPreferredSize(new Dimension(0,20));

      labelTime.setBackground(getBackground().darker());
      labelTime.setForeground(Layout.colorForeground);
      labelTime.setOpaque(true);
      labelTime.setFont(Layout.fontBold);
      labelTime.setHorizontalAlignment(JLabel.LEFT);
      labelTime.setPreferredSize(new Dimension(0,20));

      j.add(panel,new Constraints(0,6,3,1,0.0,0.0,new Insets(0,0,0,0)));
   }

   private final Simulator to_close;
   public void dispose ()  // dispose -> close
   {
      to_close.close();
      super.dispose();
   }

   // ---- Update Commands...

   public void updateLabel (final int state, final double time, final double hertz)
   {
      SwingUtilities.invokeLater(new Runnable() {
         public void run () {
      labelState.setText((state == 0? "Initialized" :
                          time == duration? "Complete" :
                          state == 1? "Paused" :
                          state == 2? "Running" : "???") + "   -   ");

      labelTime.setText(state == 0? "Game open" :
                        Layout.getTimeString(time) + "    " +
                        Layout.getHertzString(hertz));
         }
      });
   }

   public void updateChannels (final Channel[] channels)
   {
      Vector vector = new Vector();  // count teams...
      int teams = 0;

      for (int i=0; i<channels.length; ++i)
         if (!vector.contains(""+channels[i].t)) { vector.addElement(""+channels[i].t); ++ teams; }

      final int teams_ = teams;
      SwingUtilities.invokeLater(new Runnable() {
         public void run () {
      labelChannels.setText(Layout.getNumbered(channels.length,"Unit Client")+"  -  "+
                            Layout.getNumbered(teams_,"Team"));
         }
      });
   }

   public void updateControls (final Control[] controls)
   {
      SwingUtilities.invokeLater(new Runnable() {
         public void run () {
      labelControls.setText(Layout.getNumbered(controls.length,"Control Client"));
         }
      });
   }
}
