/*
   Hoverball
   Copyright (c) 2002-2021 Stefan Bornhofen, Matthias Bornhofen

   This file is part of Hoverball.

   Hoverball is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   Hoverball is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with Hoverball. If not, see <http://www.gnu.org/licenses/>.
 */
package hoverball.simulator;

import hoverball.math.Complex;
import hoverball.math.Matrix;


// ---- Unite

class Unite extends Disque
/*
   Une Unité possede deux moteurs, notamment a gauche et a droite.
*/
{
   private final double xQ;
   private final double xmX;

   Unite (int t, int n, double m, double r, Matrix X, Complex p, double L, double xQ, double xmX,
          double E, double py)
   {
      super(UNITE,t,n,m,r,0,X,p,L,"");

      this.xQ  = xQ;
      this.xmX = xmX;

      this.E = E;
      this.py = py;
   }

   // ---- Virtual (override) in Disque

   Complex xQ ()    { return new Complex(r*xQ,0); }

   // ---- Unite

   // Q [Disque]      tatsächliche Ladung Q
   double mL = 0;  // tatsächliche Wirkungskraft mL
   double mR = 0;  // tatsächliche Wirkungskraft mR
   double E = 0;   // Energievorrat
   double py = 0;  // Penalty

   double aQ = 0;  // angemeldete Aktion für Q
   double amL = 0; // angemeldete Aktion für mL
   double amR = 0; // angemeldete Aktion für mR

   Complex xmL () { return new Complex(0, r*xmX); }
   Complex xmR () { return new Complex(0,-r*xmX); }
   Complex smL () { return new Complex(1,0);      }
   Complex smR () { return new Complex(1,0);      }
}
