package hoverball.team.Barbarians_;

import java.util.Iterator;
import hoverball.*;
import hoverball.debug.*;
import hoverball.math.*;

public class Barbarian extends OrientedUnit
{
	public enum State {
		THINK, AIM, GET_BALL, RECOVER, ROAM
	}
	private State state;
	public enum TeamMess {
		IBALL, ROAMB, ROAMU, NONE
	}
	private String teamMess;
	private double Qmax, Fmax;
	private Vector target;
	private double targetDist;
	private String status;
	private double touchingDist;
	private double controlDist;
	private double controlShootDist;
	private double controlAngle;
	private double maxShotDist;
	private double minShotDist;
	private double interceptionFactorGoal;
	private double interceptionGoalRadius;
	private static boolean DEBUG = true;
	private String IDGOAL = "";

	public Barbarian(String name, int color)
	{
		super(name,color);
		state = State.THINK;
		teamMess = TeamMess.NONE.toString();
	}

	protected void loop()
	{
		// Initialization:
		sphere = new Sphere(option("world.radius"));
		Qmax = option("unit.charge.max");
		Fmax = option("unit.engine.max");
		touchingDist = 0;//option("unit.radius") + option("ball.radius");
		controlDist = touchingDist + 4;
		controlShootDist = touchingDist + 1;
		controlAngle = Math.PI / 6; // +-30°: 60° control range
		// maxShotDist = option("world.radius")*Math.PI*0.5;
		maxShotDist = sphere.rad*2*Math.PI;
		minShotDist = 0;
		interceptionFactorGoal = 0.75;
		interceptionGoalRadius = 50;

		while ( look() )
		{
			// Inner Initialization
			IDGOAL = "ball " + self.t + " 1";
			super.loopMethods();

			switch (state)
			{
			case GET_BALL:
				get_ball();
				break;
			case AIM:
				precise_kick();
				break;
			case RECOVER:
				recover();
				break;
			default:
				think();
				break;
			}

			if ( DEBUG )
			{
				targetDist = Math.floor(targetDist * 100.0) / 100.0;
				String energyStr = Double.toString(Math.floor(energy * Qmax * 100.0) / 100.0);
				debug(new Text(self.X.c, title + " (" + energyStr + ")", -2));
				debug(new Text(self.X.c, "info: " + status + " \n status " + state + " \n team: "
						+ teamMess, 2));
				// debug(new Text(self.X.c, name+" \n E=" + energyStr + "\n
				// status: " + status
				// + "\n d=" + targetDist, 1), -1);
				if ( target != null )
					debug(new Line(self.X.c, target));
			}
		}// end of while(look())
	}

	private void think()
	{
		Matrix ball;
		Matrix goal;
		target = null;
		targetDist = 0;
		status = "";

		boolean mateHasBall = false;
		// ball position and ball distance of nearest mate

		Vector mateBallPos = new Vector();
		double mateBallDist = -1;

		// run out of energy, recover:
		if ( energy <= (Qmax * 0.1) )
		{
			status = "Recovering energy";
			state = State.RECOVER;
			return;
		}

		for ( int j = 0; j < pucks.length; j++ )
		{
			Puck puck = pucks[j];
			if ( puck.id.startsWith("unit " + self.t) && !puck.id.equals(self.id)
					&& puck.message.startsWith(TeamMess.IBALL.toString()) )
			{
				String pmess = puck.message;
				double c[] = new double[3];
				int spos = 0;
				double dist;

				for ( int i = 0; i < c.length; i++ )
				{
					spos = pmess.indexOf(":", spos + 1);
					c[i] = Double.parseDouble(pmess.substring(spos + 1,
							pmess.indexOf(":", spos + 1)).replace(",", "."));
					
				}
				//System.out.println(title+": ball["+c[0]+"."+ c[1]+"."+ c[2]+"]");
				spos = pmess.indexOf(":", spos + 1);
				dist = Double.parseDouble(pmess.substring(spos + 1,pmess.indexOf(":", spos + 1)).replace(",", "."));
				//System.out.println(title+": dist "+dist);
				if ( mateBallDist == -1 || dist < mateBallDist || dist == -1 )
				{
					mateBallDist = dist;
					mateBallPos = new Vector(c[0], c[1], c[2]);
				}

				mateHasBall = true;

				//System.out.println(name+" mate("+puck.n+"): "+dist);
			}
		}

		if ( !posMap.containsKey(IDBALL) && !mateHasBall )
		{ // ball not visible
			status = "Looking for ball";
			teamMess = TeamMess.NONE.toString();
			turn(false);
			return;
		} else if ( !posMap.containsKey(IDBALL) )
		{
			status = "i dont see, but team does.";
			teamMess = TeamMess.NONE.toString();
			roam();
			return;
		}
		ball = getRelativeCoords(posMap.get(IDBALL).position);
		// debug(new Text(self.X.c,"ball "+sphere.warp(ball.c).arg(),-4));

		double ballDist = sphere.diff(self.X.c, ball.c);
		double ballAngle = (sphere.warp(ball.c)).arg();

		// broadcast ball position
		teamMess = TeamMess.IBALL + ":"
				+ String.format("%1$.2f:%2$.2f:%3$.2f", ball.c.x, ball.c.y, ball.c.z);
		teamMess = teamMess + ":" + String.format("%1$.2f:", ballDist);

		// is mate closer to ball?
		if ( mateBallDist != -1 && ballDist > mateBallDist )
		{
			status = "mate is closer to ball";
			roam();
			return;
		}

		if ( ballDist > controlDist || Math.abs(ballAngle) > controlAngle )
		{ // ball out of range
			status = "Going / turning to ball";
			goto_ball();
			return;
		}

		if ( !posMap.containsKey(IDGOAL) )
		{
			status = "Looking for goal";
			turn(true);
			return;
		} else
			goal = getRelativeCoords(posMap.get(IDGOAL).position);

		double goalAngle = (sphere.warp(goal.c)).arg();
		if ( Math.abs(goalAngle) > 0.1 )
		{
			status = "Turning to goal";
			turnto_goal();
			return;
		}

		double goalDist = sphere.diff(self.X.c, goal.c);
		if ( goalDist > maxShotDist )
		{ // too far away: "dribble" in rough direction
			status = "Passing ball closer to goal";
			action(0.1, 0, 0, teamMess);
			return;
		}

		if ( ballDist > controlShootDist )
		{
			action(-0.01, 0, 0, teamMess);
			target = ball.c;
			targetDist = sphere.diff(self.X.c, target);

		}

		if ( goalDist < minShotDist )
		{ // too close
			status = "Getting out of violation range";
			retreat();
			return;
		}

		// else: ball in control range, goal close and in front:
		// go for it!
		status = "Aiming at goal";
		precise_kick();
		return;

	} // end of think()

	/**
	 * Positions unit in front of ball
	 */
	private void goto_ball()
	{
		Matrix ball;

		if ( !posMap.containsKey(IDBALL) )
		{
			return;
		} else
			ball = getRelativeCoords(posMap.get(IDBALL).position);

		// target position: in front of ball
		Complex ballWarp = sphere.warp(ball.c);
		Complex targetWarp = Complex.sub(ballWarp, (Complex.norm(ballWarp)).mul(touchingDist));

		// position in between ball and goal, if goal
		// visible and behind ball
		Matrix goal;
		if ( posMap.containsKey(IDGOAL) )
		{
			goal = getRelativeCoords(posMap.get(IDGOAL).position);
			Complex goalWarp = sphere.warp(goal.c);
			Complex ballToGoal = Complex.sub(goalWarp, ballWarp);

			// goal behind ball
			if ( sphere.diff(self.X.c, goal.c) >= sphere.diff(self.X.c, ball.c) )
			{
				targetWarp = Complex.sub(ballWarp, (ballToGoal.norm()).mul(touchingDist));
			} else
			{// goal in between unit and ball
				double dist = Complex.vec(goalWarp, ballWarp) / ballWarp.abs();
				double securityDist = touchingDist+3;
				if ( Math.abs(dist) <= securityDist )
				{ // too close to goal
					double t = Complex.dot(goalWarp, ballWarp) / ballWarp.abs2();
					Complex closestPt = Complex.mul(ballWarp, t);
					targetWarp = Complex.add(goalWarp, Complex.sub(closestPt, goalWarp).norm().mul(
							securityDist));

				}
				debug(new Circle(goal.c, securityDist / option("world.radius")), -1);
			}

		}

		double thrust = Math.max(0.2, 1 - 10 * Math.abs(targetWarp.arg()));
		double l = -targetWarp.arg() + thrust;
		double r = targetWarp.arg() + thrust;
		target = sphere.warp(targetWarp);
		targetDist = sphere.diff(target, self.X.c);
		
		if ( targetDist > 60 && Math.abs(targetWarp.arg()) < Math.PI/12)
		{
			l = 1;
			r = 1;
		}

		/*
		 * if ( targetDist <= controlDist ) { // got ball, keep it close state =
		 * State.GET_BALL; action(-0.01, 0, 0, teamMess); target = ball.c;
		 * targetDist = sphere.diff(self.X.c, target); } else
		 */
		{
			action(0, l * Fmax, r * Fmax, teamMess);
		}

	}

	private void precise_kick()
	{
		target = null;
		Matrix ball;
		Matrix goal;
		if ( !posMap.containsKey(IDBALL) || !posMap.containsKey(IDGOAL) )
		{
			state = State.THINK;
			return;
		}
		ball = getRelativeCoords(posMap.get(IDBALL).position);
		goal = getRelativeCoords(posMap.get(IDGOAL).position);

		double goalDist = sphere.diff(self.X.c, goal.c);
		double ballDist = sphere.diff(self.X.c, ball.c);
		double ballAngle = (sphere.warp(ball.c)).arg();

		if ( ballDist > controlDist || Math.abs(ballAngle) > controlAngle )
		{
			state = State.THINK;
			return;
		}

		// TODO: needs to check if shot is possible at all by aiming
		// (ball out of angle

		Complex ballPos = sphere.warp(ball.c);

		// Unit's center of charge:
		Complex forceCenter = new Complex(0.5 * option("unit.radius"), 0);

		// direction of force from forceCenter to ball
		Complex forceDirection = (Complex.sub(ballPos, forceCenter));
		// from ball to goal:
		Complex intendedForceDirection = (Complex.sub(sphere.warp(goal.c), ballPos));

		// distance
		double l = intendedForceDirection.abs() + ballDist - option("ball.radius");
		forceDirection = forceDirection.norm(); /*immutable*/
		intendedForceDirection = intendedForceDirection.norm(); /*immutable*/
		Vector shotTarget = sphere.warp(Complex.add(forceCenter, Complex.mul(forceDirection, l)));
		// how far would the goal be missed:
		double missedBy = sphere.diff(goal.c, shotTarget);

		if ( missedBy < option("ball.radius") )
		{
			action(0.15 / 18 * (goalDist + goalDist / 6), Fmax, Fmax, teamMess);
			state = State.THINK;
		} else
		{
			double crossProd = Complex.vec(forceDirection, intendedForceDirection);
			action(0, crossProd * Fmax, -crossProd * Fmax, teamMess);
			state = State.AIM;
		}

		debug(new Line(sphere.warp(forceCenter), shotTarget));
		debug(new Line(ball.c, goal.c), -1);
	}

	// turns Unit with or without attracting the ball
	private void turn(boolean attract)
	{
		double load = (attract) ? -0.01 : 0;
		action(load, -Fmax, Fmax, teamMess);

	}

	/**
	 * Wait until energy is recovered
	 */
	private void recover()
	{
		teamMess = TeamMess.NONE.toString();
		action(0, 0, 0, teamMess);
		if ( energy >= (0.6 * Qmax) )
		{
			state = State.THINK;
			return;
		}

	}

	/**
	 * Move away from goal if too close
	 */
	private void retreat()
	{
		Matrix goal;
		if ( !posMap.containsKey(IDGOAL) )
			return;
		else
			goal = getRelativeCoords(posMap.get(IDGOAL).position);
		target = goal.c;
		targetDist = sphere.diff(self.X.c, goal.c);
		action(-0.01, -Fmax, -Fmax, teamMess);
		debug(new Circle(goal.c, minShotDist / option("world.radius")), -1);

	}

	/**
	 * Turns unit towards goal, keeps ball attracted
	 */
	private void turnto_goal()
	{
		Matrix goal;
		if ( !posMap.containsKey(IDGOAL) )
		{
			return;
		} else
		{
			goal = getRelativeCoords(posMap.get(IDGOAL).position);
			Complex x = sphere.warp(goal.c);
			target = goal.c;
			targetDist = sphere.diff(self.X.c, goal.c);
			double l = -x.arg();
			double r = x.arg();
			action(-0.02, l * Fmax, r * Fmax, teamMess);
		}
	}

	/**
	 * Attract ball and wait for it to calm down
	 */
	private void get_ball()
	{
		status = "Securing ball";
		Matrix ball;
		if ( !posMap.containsKey(IDBALL) )
		{ // if no ball visible: start searching again
			return;
		} else
		{
			ball = getRelativeCoords(posMap.get(IDBALL).position);

			double ballDist = sphere.diff(ball.c, self.X.c);
			// difference in ball distance small => ball pos. stable
			if ( ballDist > controlDist || Math.abs(ballDist - targetDist) < 0.01 )
			{
				state = State.THINK;
				return;
			}
			action(-0.01, 0, 0, teamMess);
			target = ball.c;
			targetDist = ballDist;
		}

	}

	private void roam()
	{
		Matrix ball;
		boolean mateRoamU = false, mateRoamB = false;
		double mateRoamBdist = -1;
		String puckmess;
		if ( !posMap.containsKey(IDBALL) )
		{ // if no ball visible: start searching again
			turn(false);
			return;
		}
		ball = getRelativeCoords(posMap.get(IDBALL).position);

		// another unit intercepting anything??
		/*for ( int j = 0; j < pucks.length; j++ )
		{
			Puck puck = pucks[j];
			if ( puck.id.startsWith("unit " + self.t) && !puck.id.equals(self.id) )
			{
				if ( puck.message.contains(TeamMess.ROAMB.toString()) )
				{
					
					puckmess = puck.message;
					mateRoamBdist = Double.parseDouble(puckmess.substring(puckmess
							.indexOf(TeamMess.ROAMB.toString()), puckmess.indexOf(":", puckmess
							.indexOf(TeamMess.ROAMB.toString()))));
					if ( mateRoamBdist < interceptionDistance )
					{
						mateRoamB = true;
					}
				} else if ( puck.message.contains(TeamMess.ROAMU.toString()) )
					mateRoamU = true;
			}
		}
		if ( !mateRoamB )
			interceptBallGoal(ball);
		// else if ( !mateRoamU )
		// interceptUnit(ball);
		else
		{
			state = State.THINK;
		}*/
		interceptBallGoal(ball);
	}

	private void interceptBallGoal(Matrix ball)
	{
		Matrix eGoal = null;
		for ( Iterator<String> iter = posMap.keySet().iterator(); iter.hasNext(); )
		{
			String key = (String) iter.next();
			if ( key.startsWith("ball") )
				if ( !key.startsWith("ball 0") )
					if ( !key.startsWith("ball " + self.t) )
						eGoal = getRelativeCoords(posMap.get(key).position);
		}
		if ( eGoal != null )
		{
			if ( sphere.diff(eGoal.c, ball.c) * interceptionFactorGoal < interceptionGoalRadius )
			{
				goto_between(ball, eGoal, interceptionFactorGoal);
			} else
			{
				goto_between(ball, eGoal, interceptionGoalRadius / sphere.diff(eGoal.c, ball.c));
			}
			teamMess = teamMess+TeamMess.ROAMB.toString() + String.format("%1$.2f:", targetDist);
			//state = State.ROAM;
		} else
			turn(false);
	}

	private void interceptUnit(Matrix ball)
	{

	}

	private void goto_pos(Vector pos)
	{
		Complex posWarp = sphere.warp(pos);

		double thrust = Math.max(0.2, 1 - 10 * Math.abs(posWarp.arg()));
		double l = -posWarp.arg() + thrust;
		double r = posWarp.arg() + thrust;
		target = sphere.warp(posWarp);
		targetDist = sphere.diff(target, self.X.c);

		/*if ( Math.abs(posWarp.arg()) > 100 )
		{
			l = -posWarp.arg() - thrust;
			r = -posWarp.arg() - thrust;
		} else*/
		{
			l = -posWarp.arg() + thrust;
			r = posWarp.arg() + thrust;
		}

		action(0, l * Fmax, r * Fmax, teamMess);
	}

	/**
	 * @param posA
	 *            First position.
	 * @param posB
	 *            Second position.
	 * @param factor
	 *            How much of the distance from A to B. A 0.5 would be the
	 *            middle, 0.25 is only one quarter of the way from A to B. Can
	 *            be 0-1.
	 */
	private void goto_between(Matrix posA, Matrix posB, double factor)
	{
		// position in between posA and posB
		Complex posAwarp = sphere.warp(posA.c);
		Complex posBwarp = sphere.warp(posB.c);
		Complex aToB = Complex.sub(posBwarp, posAwarp);
		Complex targetWarp = Complex.add(posAwarp, aToB.mul(factor));
		debug(new Line(self.X.c, sphere.warp(targetWarp)));
		if ( targetWarp.abs() > option("unit.radius") )
			goto_pos(sphere.warp(targetWarp));
	}
	
	private double interceptDist(Vector a, Vector b, double f)
	{
		Complex posAwarp = sphere.warp(a);
		Complex posBwarp = sphere.warp(b);
		Complex aToB = Complex.sub(posBwarp, posAwarp);
		Complex targetWarp = Complex.add(posAwarp, aToB.mul(f));
		return sphere.diff(self.X.c, sphere.warp(targetWarp));
	}
}
