package hoverball.team.Barbarians_;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import hoverball.Puck;
import hoverball.Unit;
import hoverball.math.*;

public class OrientedUnit extends Unit
{
	protected Map<String, StoredPuckAbs> posMap = new HashMap<String, StoredPuckAbs>();
	protected List<StoredPuckAbs> teamList = new ArrayList<StoredPuckAbs>();
	protected final String IDBALL = "ball 0 1";
	protected Matrix pole = new Matrix();
	protected Sphere sphere;

	public OrientedUnit(String name, int color)
	{
		super(name,color);
	}

	@Override
	protected void loop()
	{
		sphere = new Sphere(option("world.radius"));
		while ( look() )
		{
			pole = getPole();
			updateEnvironmentPositions();
		}
	}

	/**
	 * call this method from sub-classes' while(look())-loop for variable
	 * initialization
	 */
	protected void loopMethods()
	{
		pole = getPole();
		updateEnvironmentPositions();
	}

	protected void updateEnvironmentPositions()
	{
		Puck tmpPuck;
		StoredPuckAbs storedPuck;
		// go through all nodes and update stored data

		for ( int i = 0; i < pucks.length; i++ )
		{
			tmpPuck = pucks[i];
			// do we only hear this puck?
			if ( tmpPuck.X != null && !tmpPuck.id.startsWith("node") )
			{
				if ( !posMap.containsKey(tmpPuck.id) )
				{
					//System.out.println(title+": Adding puck " + tmpPuck.id + "...");
					posMap.put(tmpPuck.id, new StoredPuckAbs(tmpPuck.id,
							getAbsoluteCoords(tmpPuck.X), time));
					posMap.get(tmpPuck.id).message = tmpPuck.message;

				} else
				{
					storedPuck = posMap.get(tmpPuck.id);
					storedPuck.updatePosition(getAbsoluteCoords(tmpPuck.X), time);
					storedPuck.message = tmpPuck.message;
				}

			}

		}

		// update timestamps
		for ( Iterator<String> iterator = posMap.keySet().iterator(); iterator.hasNext(); )
		{
			String key = (String) iterator.next();
			StoredPuckAbs invalidPuck = posMap.get(key);
			Matrix relPosToPuck = getRelativeCoords(invalidPuck.position);
			boolean invalidPos = false;
			String debstr = "";
			if ( isSeeingPosition(relPosToPuck.c) )
			{
				boolean found = false;
				for ( int i = 0; i < pucks.length; i++ )
				{
					debstr = debstr +","+pucks[i].id;
					if ( pucks[i].id.equals(invalidPuck.id) )
					{
						found = true;
					}
				}
				if ( !found )
				{
					invalidPos = true;
				}
			}
			if ( invalidPos )
			{
				iterator.remove();
				//System.out.println(title + ": removing puck " + invalidPuck.id + " @ "
				//		+ sphere.warp(relPosToPuck.c) + "°"
				//		+ sphere.warp(relPosToPuck.c).arg()/Math.PI*180+" while seeing "+debstr);
			} else
				posMap.get(key).updateTimestamp(time);
		}
	}

	/**
	 * This method must be called in the beginning of the while(look()) loop
	 * before the call to updateEnvironment()
	 *
	 * @return relativ position to node 0 1
	 */
	protected Matrix getPole()
	{
		Matrix pole;
		Puck node = puck(NODE, 0, 1);
		if ( node != null )
			pole = node.X;
		else
		{
			node = puck(NODE, 0, 6);
if (node == null) return new Matrix();  // fix NullPointerException Bug!
			pole = node.X.neg();
			pole = new Matrix(pole.a,pole.b.neg(),pole.c); /*immutable*/
			// TODO this should be fixed...
		}
		return pole;
	}

	protected Matrix getRelativeCoords(Matrix m)
	{
		return new Matrix(pole).mulL(m);
	}

	protected Matrix getAbsoluteCoords(Matrix m)
	{
		return Matrix.inv(pole).mulL(m);
	}

	protected ArrayList<StoredPuckAbs> getTeamPucks()
	{
		// TODO: only seen pucks are listed
		ArrayList<StoredPuckAbs> list = new ArrayList<StoredPuckAbs>();
		for ( Iterator<String> iterator = posMap.keySet().iterator(); iterator.hasNext(); )
		{
			String key = (String) iterator.next();
			StoredPuckAbs store = posMap.get(key);
			if ( store.id.startsWith("unit " + self.t) )
			{
				list.add(store);
			}
		}
		return list;
	}

	protected boolean isSeeingPosition(Vector v)
	{
		Complex vWarp = sphere.warp(v);
		if ( Math.abs(vWarp.arg()) / Math.PI * 180 > 89 )
			return false;
		else
			return true;
	}
}
