package hoverball.team.FantasticThree_;
/* Hoverball Competition 2007
 *
 * Team   : Fantastic Three
 * Author : Armin Hornung
 */

import java.util.ArrayList;

import hoverball.*;
import hoverball.debug.*;
import hoverball.math.*;

public class FantasticThree extends Team
{
  // ---- Application

  public static void main (String[] args)
  {
    Application.setBounds();

    new FantasticThree("Fantastic Three").show();
  }

  // ---- Constructor

  public FantasticThree() { this(null); }
  public FantasticThree(String team) { this(team,0x8822AA); }
  public FantasticThree(int color) { this(null,color); }
  public FantasticThree(String team, int color)
  {
    super((team == null || team == "")? "Fantastic Three" : team,
       "1.0 (October 2007)","Armin Hornung","...");

    add(new FPlayer(null, "One", color, 0));
    add(new FPlayer(null, "Two", color, 1));
    add(new FPlayer(null, "Three", color, 2));
  }
}

// *************************************************************************
// Spieler
// *************************************************************************

class FPlayer extends Unit
{
   public FPlayer(String team, String name, int color, int id) {
	   super(team, name, color);
	   state = State.THINK;
	   this.id = id;
           this.name = name;
   }
   // available states of Player:
   public enum State {THINK, AIM, GET_BALL, RECOVER, GUARD, PASS }
   public enum Role{IDLE, DEFENDER, ATTACKER, ASSIST}

   private String name;
   private State state;
   private double Qmax, Fmax;
   private Sphere sphere;
   private double prevBallDist;
   private String status;
   private Role role;
   // "constants"
   private double touchingDist;
   private double securityDist;
   private double controlDist;
   private double controlShootDist;
   private double controlAngle;
   private double maxShotDist;
   private int id;
   private double load;
   private double F_l;
   private double F_r;

   // seen pucks
   private Vector enemyGoal = null;
   private Vector ownGoal = null;
   private Vector ball = null;
   private Vector[] teamPlayers = new Vector[3];
   private ArrayList<Vector> enemyPlayers = new ArrayList<Vector>();
   private int[] teamStatus = new int[3];

   // saved Positions
   private Vector enemyGoalGlobal = null;
   private Vector ownGoalGlobal = null;
   private Vector ballGlobal = null;


   // global Coordinate system
   private Matrix pov;

   // distances:
   private double ownGoalDist[] = new double[3];
   private double enemyGoalDist[] = new double[3];
   private double ballDist[] = new double[3];
	private String ballCoordMsg;
	private String ownGoalCoordMsg;
	private String enemyGoalCoordMsg;
	private int attackerId;
	private int defenderId;
	private int assistId;

   public void loop()
   {
	   //Initialization:
	  sphere = new Sphere(option("world.radius"));
      Qmax = option("unit.charge.max");
      Fmax = option("unit.engine.max");
      touchingDist = option("unit.radius")+2*option("ball.radius");
      securityDist = touchingDist + 6;
      controlDist = touchingDist + 3;
      controlShootDist = touchingDist + 1;
      controlAngle = Math.PI/6; // +-30°: 60° control range
      //maxShotDist = option("world.radius")*Math.PI*0.5;
      maxShotDist = 60;
      role = Role.IDLE;
      while(look()){
    	  // first: update state
    	  update_world();

    	  switch (state) {
			case GET_BALL:
				get_ball();
			break;
			case PASS:
				pass();
				break;
			case RECOVER:
				recover();
				break;
			default:
				think();
				break;
		 }

    	  // do action:
    	  String message = id+" "+ getMsgCoords(getAbsoluteCoords(self.X.c));
    	  message += " "+ballCoordMsg+" "+ownGoalCoordMsg
    	  			+ " "+enemyGoalCoordMsg + " "+getMsgState();
    	  action(load,F_l,F_r,message);
    	  String energyStr = Double.toString(Math.floor(energy*Qmax*100.0) /100.0);
    	  debug(new Text(self.X.c, name+": "+role+" \n E="+energyStr+"\n status: "+status, 1), -1);


      } // end of while(look())

   } // end of loop()

   private void pass() {
	   if (ball == null){
		   state = State.THINK;
		   return;
	   }

	   double ballAngle = angle(ball);
	   if (ballDist[id] > controlDist || Math.abs(ballAngle) > controlAngle
			   || sphere.warp(ball).x <= 0){ // ball out of range
		   status = "Going / turning to ball";
		   state = State.THINK;
		   goto_ball();
		   return;
	   }
	   Complex assistPos = sphere.warp(teamPlayers[assistId]);
	   if (Math.abs(angle(assistPos)) > 0.02)
		   turn(assistPos, true);
	   else{
		   load = 0.2/33*sphere.diff(self.X.c, teamPlayers[assistId]);
		   state = State.THINK;
		   return;
	   }


}

private String getMsgState() {
      if (state == State.RECOVER)
    	  return "-1";
      else
    	  return "0";
   }

   private int getStateFromMsg(String message){
	   String[] tokens = message.split("\\s");
		return Integer.parseInt(tokens[13]);
   }

private void update_world() {

	   updatePOV();

	   ownGoal = null;
	   enemyGoal = null;
	   ball = null;
	   for (int i = 0; i < 3; i++) {
		   teamPlayers[i] = null;
		   teamStatus[i] = 0;

	   }
	   enemyPlayers.clear();

	   Vector ballReceived = null;
	   Vector ownGoalReceived = null;
	   Vector enemyGoalReceived = null;

	   // go through array of seen pucks
	   for (int i = 0; i < pucks.length; i++) {
		   //if (pucks[i].X != null)
		   { // puck seen!
			   Puck curr_puck = pucks[i];
			   if(curr_puck.equals(BALL)){
				   if (curr_puck.t == 0){
					   ball = new Vector(curr_puck.X.c);
					   ballGlobal = getAbsoluteCoords(ball);
				   }
				   else if (curr_puck.t == self.t){
					   ownGoal = new Vector(curr_puck.X.c);
				   	   ownGoalGlobal = getAbsoluteCoords(ownGoal);
				   }
				   else{
					   enemyGoal = new Vector(curr_puck.X.c);
					   enemyGoalGlobal = getAbsoluteCoords(enemyGoal);
				   }

			   }
			   else if (curr_puck.equals(UNIT)){
				   // own team, parse message
				   if (curr_puck.t == self.t && curr_puck.message.length()>0){
					   int playerId = getIdFromMsg(curr_puck.message);
					   if (curr_puck.X != null) // team player seen
						   teamPlayers[playerId] = new Vector(curr_puck.X.c);
					   else // only heard team member
						   teamPlayers[playerId] = getRelativeCoords(getVectorFromMsg(curr_puck.message, 0));

					   Vector rBall = getVectorFromMsg(curr_puck.message, 1);
				   		if (!rBall.equals(0, 0, 0))
			   				ballReceived = rBall;

				   		Vector rOwnGoal = getVectorFromMsg(curr_puck.message, 2);
				   		if (!rOwnGoal.equals(0, 0, 0))
				   			ownGoalReceived = rOwnGoal;

				   		Vector rEnGoal = getVectorFromMsg(curr_puck.message, 3);
				   		if (!rEnGoal.equals(0, 0, 0))
				   			enemyGoalReceived = rEnGoal;

				   		teamStatus[playerId] = getStateFromMsg(curr_puck.message);

				   }
				   else if (curr_puck.t != self.t && curr_puck.X != null){ // enemy team
					   enemyPlayers.add(new Vector(curr_puck.X.c));


				   }

			   }
		   }
	   }


	   // if a puck was not seen, only send out 0's:
	   ballCoordMsg = getMsgCoords(new Vector(0,0,0));
	   ownGoalCoordMsg = getMsgCoords(new Vector(0,0,0));
	   enemyGoalCoordMsg = getMsgCoords(new Vector(0,0,0));

	   // if ball / goals not seen: use from memory or received!
	   if (ball == null){
		   if (ballReceived != null)
			   ball = getRelativeCoords(ballReceived);
		   else if (ballGlobal != null)
			   ball = getRelativeCoords(ballGlobal);
	   }
	   else
		   ballCoordMsg = getMsgCoords(ballGlobal);

	   if (ownGoal == null){
		   if (ownGoalReceived != null)
			   ownGoal = getRelativeCoords(ownGoalReceived);
		   else if (ownGoalGlobal != null)
			   ownGoal = getRelativeCoords(ownGoalGlobal);
	   }
	   else
		   ownGoalCoordMsg = getMsgCoords(ownGoalGlobal);

	   if (enemyGoal == null){
		   if (enemyGoalReceived != null)
			   enemyGoal = getRelativeCoords(enemyGoalReceived);
		   else if (enemyGoalGlobal != null)
			   enemyGoal = getRelativeCoords(enemyGoalGlobal);
	   }
	   else
		   enemyGoalCoordMsg = getMsgCoords(enemyGoalGlobal);

	   // save distances:
	   for (int i = 0; i < 3; i++) {
		   ownGoalDist[i] = 99999999;
		   ballDist[i] = 99999999;
		   if (teamPlayers[i] != null){
			   if (ownGoal != null)
				   ownGoalDist[i] = sphere.diff(teamPlayers[i], ownGoal);
			   if (ball != null)
				   ballDist[i] = sphere.diff(teamPlayers[i], ball);
			   if (enemyGoal != null)
				  enemyGoalDist[i] = sphere.diff(teamPlayers[i], enemyGoal);
		   }
	   }

	   /**
	   if (id == 1)
		   debug(new Circle(teamPlayers[0], 0.1));
	   else if (id == 0)
		   debug(new Circle(teamPlayers[1], 0.1));

	   debug(new Circle(ball, 0.05), -1);
	   debug(new Circle(ownGoal, 0.05));
	   debug(new Circle(enemyGoal, 0.05), -1);
	   **/

   }


private void think(){
	   status = "";
	   load = 0;
	   F_l = 0;
	   F_r = 0;

	   role = Role.IDLE;
	   // run out of energy, recover:
	   if (energy <= (Qmax* 0.1)){
		   status = "Recovering energy";
		   state = State.RECOVER;
		   return;
	   }

	   if (ball == null){ // ball not visible
		   status = "Looking for ball";
		   turn(false);
		   return;
	   }

	   double ballAngle = angle(ball);


	   // get id of attacker
	   double minDist = 999999;

	   for (int i = 0; i < 3; i++) {
		   if (teamStatus[i] >= 0 && ballDist[i] < minDist){
			   minDist = ballDist[i];
			   attackerId = i;
		   }
	   }
	   // get id of defender
	   minDist = 999999;
	   for (int i = 0; i < 3; i++) {
		   if (i != attackerId && enemyGoalDist[i] < minDist){
			   minDist = enemyGoalDist[i];
			   defenderId = i;
		   }
	   }
	   assistId = 0;
	   while(assistId == defenderId || assistId == attackerId)
		   assistId++;


	   if (id == attackerId)
		   role = Role.ATTACKER;
	   else if (id == defenderId)
		   role = Role.DEFENDER;
	   else
		   role = Role.ASSIST;



	   if (role == Role.DEFENDER){
		   if (enemyGoal == null){
			   turn(false);
			   return;
	   		}
		   Complex goalToBall = vectorDiff(enemyGoal, ball);
		   double dist = Math.max(touchingDist + 3,Math.min(goalToBall.abs()/2, securityDist+4));
		   goalToBall = goalToBall.norm(); /*immutable*/
		   Complex target = sphere.warp(enemyGoal).add(goalToBall.mul(dist));
		   if (sphere.diff(sphere.warp(target), self.X.c) < touchingDist +2)
			   // TODO: check if goal is covered!
			   turn(sphere.warp(ball), false);
		   else{
			   moveToPos(target);
		   }

		   debug(new Circle(enemyGoal, dist/sphere.rad),-1);
	   }

	   // target position: on flank of attacker, near goal
	   else if (role == Role.ASSIST){
		   if (ownGoal == null || teamPlayers[attackerId] == null){
			   turn(false);
			   return;
		   }

		   Vector attackerToGoal = Vector.sub(ownGoal, teamPlayers[attackerId]);

		    Complex target = Complex.mul(new Complex(Vector.vec(attackerToGoal, ownGoal).norm()), 20).add(sphere.warp(ownGoal));
		   //Complex attackerToGoal = vectorDiff(teamPlayers[0], ownGoal);
		   //Complex target = sphere.warp(ownGoal).add(attackerToGoal.mul(10));
		   if (sphere.diff(sphere.warp(target), self.X.c) < touchingDist+2){
		    	action(0,0,0);
		    	turn(sphere.warp(ball), false);
		   }
		   else{
			   moveToPos(target);
		   }


	   }
	   else if (role == Role.ATTACKER){
		   if (ballDist[id] > controlDist || Math.abs(ballAngle) > controlAngle
				   || sphere.warp(ball).x <= 0){ // ball out of range
			   status = "Going / turning to ball";
			   goto_ball();
			   return;
		   }

		   if (ownGoal == null){
			   status = "Looking for goal";
			   turn(true);
			   return;
		   }

		   double goalAngle = angle(ownGoal);
		   if (Math.abs(goalAngle) > 0.03){
			   status = "Turning to goal";
			   search_goal();
			   return;
		   }

		   if (ownGoalDist[id] > maxShotDist){ // too far away: "dribble" in rough direction
			   status = "Passing ball closer to goal";
			   load = 0.2/16*ownGoalDist[id];
			   return;
		   }

		   if (ballDist[0] > controlShootDist){
			   load = -0.1;

		   }

		   // else: ball in control range, goal close and in front:
		   // go for it!
		   status = "Aiming at goal";
		   precise_kick();
		   return;
	   }



   }

   // turns Unit with or without attracting the ball
   private void turn(boolean attract) {
	   load = (attract)? -0.02 : 0;
	   F_l = -Fmax;
	   F_r = Fmax;

   }

   // turn to a vector position
   private void turn(Complex c, boolean attract){
	   load = (attract)? -0.01 : 0;
	   double angle = angle(c);
	    F_l = -100*Fmax*angle/Math.PI;
	    F_r = -F_l;

   }
   /**
    * Wait until energy is recovered
    */
   private void recover(){
	   load = 0;
	   F_l = 0;
	   F_r = 0;
	   if (energy >= (0.5* Qmax)){
		   state = State.THINK;
		   return;
	   }

   }

   private void precise_kick() {
	   if (ball == null || ownGoal == null){
		   state = State.THINK;
		   return;
	   }
	   double goalDist = sphere.diff(self.X.c, ownGoal);
	   double ballDist = sphere.diff(self.X.c, ball);

	   if (ballDist > controlDist){
		   state = State.THINK;
		   return;
	   }




	   Complex ballPos = sphere.warp(ball);

	   // Unit's center of charge:
	   Complex forceCenter = new Complex(0.5*option("unit.radius"), 0);

	   // direction of force from forceCenter to ball
	   Complex forceDirection = (Complex.sub(ballPos, forceCenter));
	   // from ball to goal:
	   Complex intendedForceDirection = (Complex.sub(sphere.warp(ownGoal), ballPos));

	   // distance
	   double l = intendedForceDirection.abs() + ballDist - option("ball.radius");
	   forceDirection = forceDirection.norm(); /*immutable*/
	   intendedForceDirection = intendedForceDirection.norm(); /*immutable*/
	   Vector shotTarget = sphere.warp(Complex.add(forceCenter, Complex.mul(forceDirection, l)));

	   // TODO: needs to check if shot is possible at all by aiming
	   // Ball can be shot in angle range between radius of forceCenter

	   // TODO: passing, if path to goal is blocked and ASSIST in range
	   boolean blocked = false;

	   for (int i = 0; i < enemyPlayers.size(); i++){
		   double distance = Math.abs(distPtVec(sphere.warp(enemyPlayers.get(i)), ballPos, intendedForceDirection));
		   if (distance < touchingDist
				   && sphere.diff(ownGoal, self.X.c) > sphere.diff(enemyPlayers.get(i), self.X.c)){
			  blocked = true;
		   }
	   }

	   Vector assistPos = teamPlayers[assistId];
	   if (blocked && sphere.diff(assistPos, ownGoal) < maxShotDist){
		   state = State.PASS;
		   status = "Passing...";
		   turn(sphere.warp(assistPos), true);
		   return;
	   }


	   // how far would the goal be missed:
	   double missedBy = sphere.diff(ownGoal, shotTarget);


	   if (missedBy < 2*option("ball.radius")*0.98){
		   load = 0.2/20*goalDist;
		   F_l = Fmax;
		   F_r = Fmax;
		   state = State.THINK;
	   }
	   else{
		   double crossProd = Complex.vec(forceDirection, intendedForceDirection);
		   load = 0;
		   F_l = crossProd*Fmax;
		   F_r = -crossProd*Fmax;
		   state = State.AIM;
	   }



	   debug(new Line(sphere.warp(forceCenter), shotTarget));
	   debug(new Line(ball, ownGoal),-1);
}

   /**
    * Attract ball and wait for it to calm down
    */
	private void get_ball() {
		status = "Securing ball";
		if (ball == null){  		// if no ball visible: start searching again
			return;
		}
		else{
			// difference in ball distance small => ball pos. stable
			if (ballDist[id] > controlDist || Math.abs(ballDist[id] - prevBallDist) < 0.2){
				state = State.THINK;
				return;
			}
			prevBallDist = ballDist[id];
			load = -0.01;
			F_l = 0;
			F_r = 0;
		}

	}


	/**
	 * Turns unit towards goal, keeps ball attracted
	 */
	private void search_goal() {
		if (ownGoal == null){
			return;
		}
		else{
			Complex x = sphere.warp(ownGoal);
			turn(x, true);
		}
	}

	/**
	 * Positions unit in front of ball
	 */
	private void goto_ball() {
		if (ball == null){
			return;
		}

  	  	if (ballDist[id] <= controlDist){ // ball close, keep it!
  	  		state = State.GET_BALL;
  	  		load = -0.01;
  	  		prevBallDist = ballDist[id];
  	  		return;
		}

		// target position: in front of ball
		Complex ballWarp = sphere.warp(ball);
		Complex targetWarp = Complex.sub(ballWarp, (Complex.norm(ballWarp)).mul(touchingDist));


		// position in between ball and goal, if goal
		// visible and behind ball
		if (ownGoal != null){
			Complex goalWarp = sphere.warp(ownGoal);
			Complex ballToGoal = Complex.sub(goalWarp, ballWarp);

			// goal behind ball
			if ((Complex.dot(goalWarp, ballWarp) > 0 && // both in front
					sphere.diff(self.X.c, ownGoal) >= sphere.diff(self.X.c, ball))){
				targetWarp = Complex.sub(ballWarp, (ballToGoal.norm()).mul(touchingDist));
			}

		}
		moveToPos(targetWarp);


	}

	/**
	 * Moves unit to a target position
	 * (calculates thrust for engines)
	 * avoiding all goals
	 *
	 * @param targetWarp
	 */
	private void moveToPos(Complex targetWarp){
		Vector target = sphere.warp(targetWarp);
	  	double targetDist = sphere.diff(target, self.X.c);


	  	// at least one goal in between unit and target
		if (ownGoalDist[id] < targetDist || enemyGoalDist[id] < targetDist){
			Vector avoid;
			if (enemyGoal != null && ownGoal == null)
				avoid = enemyGoal;
			else if (enemyGoal == null && ownGoal != null)
				avoid = ownGoal;
			else if(enemyGoal != null && enemyGoalDist[id] < ownGoalDist[id])
				avoid = enemyGoal;
			else if (ownGoal != null && ownGoalDist[id] < enemyGoalDist[id])
				avoid = ownGoal;
			else
				return;

			Complex avoidWarp = sphere.warp(avoid);
			double dist = distPtVec(avoidWarp,new Complex(0,0), targetWarp);

			if (Math.abs(dist) <= securityDist && avoidWarp.x > 0){ // too close to goal and moving towards
				double t = Complex.dot(avoidWarp, targetWarp) / targetWarp.abs2();
				Complex closestPt = Complex.mul(targetWarp, t);
				targetWarp = Complex.add(avoidWarp, Complex.sub(closestPt, avoidWarp).norm().mul(securityDist));
				target = sphere.warp(targetWarp);
				targetDist = sphere.diff(self.X.c, target);
				debug(new Circle(avoid, securityDist/sphere.rad), -1);
			}
		}
		double angle = angle(targetWarp);

		if (Math.abs(angle)>0.3)
			turn(targetWarp, false); // Ziel nicht vor mir -> dreh dich

	    else {
	    	// throttle down: distance in betw. circumerferences.
	    	double d = Math.min(8,Math.abs(targetDist - touchingDist));
	    	F_l = Math.max(0.1, d/8) * Fmax;
	    	F_r = F_l;
	    }

//		double thrust = Math.max(0.2,1-10*Math.abs(targetWarp.arg()));
//		double l = -targetWarp.arg() + thrust;
//  	  	double r =  targetWarp.arg() + thrust;
//
//
//  	  	F_l = l*Fmax;
//		F_r = r*Fmax;


  	    debug(new Line(self.X.c, target));


	}

	/**
	 * Distance of Point from Vector in plane
	 * @param point
	 * @param vector
	 * @return
	 */
	private double distPtVec(Complex point, Complex origin, Complex vector) {
		Complex direction = Complex.sub(point, origin);
		return Complex.vec(direction,vector) / vector.abs();
	}

	/**
	 * Helper function, calculate Complex vector from one 3D pt to another
	 */
	Complex vectorDiff(Vector v1, Vector v2){
		Complex v1w = sphere.warp(v1);
		Complex v2w = sphere.warp(v2);
		v2w = v2w.sub(v1w); /*immutable*/
		return v2w;
	}

	/**
	 * update global coordinate system
	 */
	private void updatePOV() {
		Puck node = puck (NODE, 0, 2);
		if (node != null){
			pov = new Matrix(node.X);
		}
		else {

			node = puck (NODE, 0, 5);
			pov = Matrix.neg(node.X);
			// HACK to fix coordinate system
			pov = new Matrix(pov.a,pov.b.neg(),pov.c); /*immutable*/  // pov.b = pov.b.neg();
		}
//		debug(new Circle(pov.c, 0.01));
//		debug(new Line(pov.c, pov.a));
//		debug(new Line(pov.c, pov.b), -1);
	}

	private Matrix getRelativeCoords(Matrix m) {
		return new Matrix(pov).mulL(m);
	}

	private Matrix getAbsoluteCoords(Matrix m) {
		return Matrix.inv(this.pov).mulL(m);
	}
	private Vector getAbsoluteCoords(Vector v) {
		return Vector.mul(v,Matrix.inv(this.pov));
	}

	private Vector getRelativeCoords(Vector v) {
		return Vector.mul(v, pov);
	}
	/**
	 * returns coordinates of a vector encoded in a string:
	 * "x y z" with each 2 dec. after .
	 */
	private String getMsgCoords(Vector v){
		String ret = "";
		if (v == null)
			return "0.00 0.00 0.00";
		ret +=Double.toString(Math.floor(v.x*100.0) /100.0)+" ";
		ret +=Double.toString(Math.floor(v.y*100.0) /100.0)+" ";
		ret +=Double.toString(Math.floor(v.z*100.0) /100.0);
		return ret;
	}

	private int getIdFromMsg(String message) {
		String[] tokens = message.split("\\s");
		return Integer.parseInt(tokens[0]);
	}

	private Vector getVectorFromMsg(String message, int index){
		String[] tokens = message.split("\\s");
		int startIdx = 1+ index*3;
		double ret_x = Double.parseDouble(tokens[startIdx++]);
		double ret_y = Double.parseDouble(tokens[startIdx++]);
		double ret_z = Double.parseDouble(tokens[startIdx++]);
		Vector ret = new Vector(ret_x,ret_y,ret_z);

		return ret;

	}
	private double angle(Complex c){
		double ang = c.arg();
		if (ang > Math.PI)
			ang-=2*Math.PI;
	    else if ( ang < -Math.PI)
	    	ang+=2*Math.PI;

		return ang;
	}
	private double angle(Vector v){
		Complex c = sphere.warp(v);
		return angle(c);
	}
}
