/**
 *
 */
package hoverball.team.TribalQueens_;

import hoverball.math.Vector;

/**
 *
 */
public class Defenseur extends Role {

	/**
	 * La distance à partir de laquelle on considère qu'on est bien placé
	 */
	private double distanceDefense = 5;
	/**
	 * Distance ° laquelle on fonce sur le ballon
	 */
	private double distanceAgression = 30;
	/**
	 * Distance ° laquelle on considere qu'un joueur est proche du but
	 */
	private double distanceJoueurProche = 20;
	/**
	 * Distance ° laquelle on considere que la balle est éloignée du but
	 */
	private double distanceBalleEloigne = 120;
	/**
	 * Distance maximale ou l'on effectue un degagement
	 */
	private double distanceDegagement = 30;
	/**
	 * Angle de précision pour la charge
	 */
	private double angleCharge = Math.PI/16;


	/**
	 * @param joueur
	 * @param terrain
	 */
	public Defenseur(TribalQueen joueur, Environnement terrain) {
		super(joueur, terrain);
		// TODO Auto-generated constructor stub
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see tribalQueens.Role#reflechir()
	 */
	public void reflechir() {
		critique = false;

		//si la balle est proche du but
		if (terrain.distance(joueur.getButAdverse(), joueur.getBalle()) < distanceAgression) {
			//on va dessus
			allerA(joueur.getBalle(), false);
			//si on est suffisament proche
			if (terrain.distance(joueur.getBalle(), joueur.getJoueur()) < 10) {
				//on l'attire
				if ((distancePartenairePlusProcheBalle() < 5) && (distanceAdversairePlusProcheBalle() < 5)) {
					envoyer(100);
					chargerJoueur(adversairePlusProcheBalle());
				} else {
					if (distancePartenairePlusProcheBalle() > distanceAdversairePlusProcheBalle()) {
						attraper(terrain.distance(joueur.getBalle(), joueur.getJoueur()));
						//si il n'y a pas de but adverse devant on la degage
						if (!devant(joueur.getButAdverse())) {
							if (distanceAdversairePlusProcheBalle() < distanceDegagement) {
								envoyer(100);
							}
						}
					}
				}
			}
		} else {
			if ((terrain.distance(joueur.getButAdverse(), joueur.getBalle()) > distanceBalleEloigne) && (distanceAdversairePlusProcheButAdverse() < distanceJoueurProche)) {
				chargerJoueur(adversairePlusProcheButAdverse());
			} else {
				// on calcul le milieu entre la balle et le but adverse
				Vector milieu = terrain.milieu(joueur.getButAdverse(), joueur
						.getBalle());
				//si le ballon est trop loin, on evite de trop s'ecarter du but
				if (terrain.distance(joueur.getButAdverse(), joueur.getBalle()) > 70) {
					milieu = terrain.milieu(milieu, joueur.getButAdverse());
				}
				// si on est loin du milieu
				if (distance(milieu) > distanceDefense) {
					// on va au milieu comme un bourrin
					allerA(milieu, false);
				} else {
					// si on est bien placé, on se tourne vers la balle
					tourner(angle(joueur.getBalle()));
				}
			}
		}
	}

	/**
	 * Permet de connaitre la distance entre le ballon et du coequipier le plus proche (hormis moi)
	 * @return la distance entre le ballon et le coequipier le plus proche (hormis moi)
	 */
	public double distancePartenairePlusProcheBalle() {
		Vector[] Partenaires = joueur.getEquipiers();
		double distance = 10000;
		for (int i=0;i<Partenaires.length;i++) {
			if (Partenaires[i] != joueur.getJoueur() && (terrain.distance(Partenaires[i], joueur.getBalle()) < distance)) {
				distance = terrain.distance(Partenaires[i], joueur.getBalle());
			}
		}
		return distance;
	}

	/**
	 * Permet de connaitre la distance entre le ballon et de l'adversaire le plus proche
	 * @return la distance entre le ballon et l'adversaire le plus proche
	 */
	public double distanceAdversairePlusProcheBalle() {
		Vector[] Adversaires = joueur.getAdversaires();
		double distance = 10000;

		for (int i=0;i<Adversaires.length;i++) {
			if (terrain.distance(Adversaires[i], joueur.getBalle()) < distance) {
				distance = terrain.distance(Adversaires[i], joueur.getBalle());
			}
		}
		return distance;
	}

	/**
	 * Permet de connaitre la distance entre le but adverse et de l'adversaire le plus proche
	 * @return la distance entre le but adverse et l'adversaire le plus proche
	 */
	public double distanceAdversairePlusProcheButAdverse() {
		Vector[] Adversaires = joueur.getAdversaires();
		double distance = 10000;

		for (int i=0;i<Adversaires.length;i++) {
			if (terrain.distance(Adversaires[i], joueur.getButAdverse()) < distance) {
				distance = terrain.distance(Adversaires[i], joueur.getButAdverse());
			}
		}
		return distance;
	}

	/**
	 * Permet de retouner l'adversaire le plus proche de la balle
	 * @return le vecteur de l'adversaire le plus proche de la balle
	 */
	public Vector adversairePlusProcheBalle() {
		Vector[] Adversaires = joueur.getAdversaires();
		double distance = 10000;
		Vector adversaire = new Vector();

		for (int i=0;i<Adversaires.length;i++) {
			if (terrain.distance(Adversaires[i], joueur.getBalle()) < distance) {
				distance = terrain.distance(Adversaires[i], joueur.getBalle());
				adversaire = Adversaires[i];
			}
		}
		return adversaire;
	}

	/**
	 * Permet de retouner l'adversaire le plus proche du but adverse
	 * @return le vecteur de l'adversaire le plus proche du but adverse
	 */
	public Vector adversairePlusProcheButAdverse() {
		Vector[] Adversaires = joueur.getAdversaires();
		double distance = 10000;
		Vector adversaire = new Vector();

		for (int i=0;i<Adversaires.length;i++) {
			if (terrain.distance(Adversaires[i], joueur.getButAdverse()) < distance) {
				distance = terrain.distance(Adversaires[i], joueur.getButAdverse());
				adversaire = Adversaires[i];
			}
		}
		return adversaire;
	}

	/**
	 * Permet de charger un joueur
	 * @param joueur
	 */
	public void chargerJoueur(Vector joueur1) {
		Vector charge = Vector.add(joueur.getButAdverse(),joueur1).mul(1.5);
		allerA(charge, false);
		if (distance(charge) < 5) {
			if (terrain.angle(joueur1) < angleCharge) {
				//System.out.println("on Fonce !");
				if (joueur.getEnergy() > 0.4) {
					// on calcule la puissance du moteur de droite
					moteurDroit = 100;
					// le moteur gauche est à la même puissance que le moteur droit
					moteurGauche = 100;
				}
			} else {
				tourner(terrain.angle(joueur1));
			}
		}
	}

}
