/**
 * 
 */
package hoverball.team.TribalQueens_;

import hoverball.math.Complex;
import hoverball.math.Sphere;
import hoverball.math.Vector;

/**
 * @author detantxavi
 *
 */
public class Environnement {

	/**
	 * Représente le terrain de jeu.
	 */
	private Sphere sphere;
	
	private double rayonBalle;
	
	/**
	 * Constructeur de l'environnement
	 * @param rayonSphere Le rayon du monde
	 */
	public Environnement(double rayonSphere,double rayonBalle) {
		super();
		this.sphere = new Sphere(rayonSphere);
		this.rayonBalle = rayonBalle;
	}
	
	/**
	 * Retourne la distance entre deux points du monde
	 * @param point1 Le 1er point
	 * @param point2 Le second point
	 * @return La distance entre les deux points
	 */
	public double distance(Vector point1, Vector point2) {
		return(sphere.diff(point1, point2));
	}
	
	/**
	 * Retourne l'angle sous lequel est vu un point du monde compris entre -Pi et Pi.
	 * 0 => en face
	 * -Pi/2 => à droite
	 * PI/2 => à gauche
	 * Pi ou -Pi => derrière
	 * @param point1 Le point qu'on regarde 
	 * @return L'angle sous lequel est vu le point compris entre -Pi et Pi
	 */
	public double angle(Vector point1) {
		// on recupère l'angle du point
		double ang = sphere.warp(point1).arg();
		// si on est au dessus de pi
		if (ang > Math.PI) {
			// alors on retourne entre -Pi et Pi
			ang-=2*Math.PI;
		} else if ( ang < -Math.PI) {
			// de même si on est en dessous de -Pi
	    	ang+=2*Math.PI;
		}
		
		// on retourne l'angle ainsi trouvé
		return ang;
	}
	
	/**
	 * Retourne le point situé au milieu de deux points
	 * @param point1 Le 1er point
	 * @param point2 Le second point
	 * @return Le point situé au milieu des deux autres
	 */
	public Vector milieu(Vector point1, Vector point2) {
		return(Vector.add(point1,point2).mul(0.5));
	}
	
	/**
	 * @return Le rayon du monde
	 */
	public double getRayon(){
		return sphere.rad;
	}

	/**
	 * @return the rayonBalle
	 */
	public double getRayonBalle() {
		return rayonBalle;
	}

	/**
	 * Wrapper of sphere function
	 * @param v
	 * @return
	 */
	public Complex warp(Vector v) {
		return this.sphere.warp(v);
	}
	
	/**
	 * Wrapper of sphere function
	 * @param c
	 * @return
	 */
	public Vector warp(Complex c) {
		return this.sphere.warp(c);
	}
	
}
