/**
 * 
 */
package hoverball.team.TribalQueens_;

import hoverball.math.Complex;
import hoverball.math.Vector;

/**
 * @author detantxavi
 * 
 */
public class Relais extends Defenseur {

	/**
	 * distance à partir de laquelle on considère qu'un joueur est proche de la balle
	 */
	private double distanceProximite = 20;

	/**
	 * IMPORTANT : joue sur le mode défense / attaque
	 * distance en deça de laquelle la balle est considérée comme trop proche du but.
	 */
	private double distanceDefensive = 70;
	
	/**
	 * IMPORTANT : influe formetement sur la position du joueur
	 * Distance du point de démarcation par rapport au but
	 */
	private double distanceDemarcationBut = 30;
	
	/**
	 * Distance tolérée par rapport au point de demarcation (au-delà on se replace)
	 */
	private double toleranceDemarcation = 20;
	
	/**
	 * Distance en delà de laquelle on fonce sur la cible pour lui exploser la tronche
	 */
	//private double distanceDefonce = 10;
	
	/**
	 * Distance en deça de laquelle on considère que l'attaquant adverse est déjà sur la balle
	 */
	private double distanceAttaquantPossedeBalle = 10;
	
	//private double distanceMinimaleBut = 50;

	//TODO à utiliser
	//private double distanceDemarcationAdversaire = 30;
	
	
	//private long millisecondsBeforeUpdate = 2000;
	
	private double distancePositionDefense = 5;
	
	/**
	 * @param joueur
	 * @param terrain
	 */
	public Relais(TribalQueen joueur, Environnement terrain) {
		super(joueur, terrain);
		// TODO Auto-generated constructor stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see tribalQueensV2.Role#reflechir()
	 */
	@Override
	public void reflechir() {
		critique = false;
		
		//Si un joueur ennemi à la balle et qu'il n'y a pas d'allier dans sa sphère d'influence, le joueur devient un défenseur
		if ( (this.joueur.distancePlusProcheAdversaire(this.joueur.getBalle()) < this.distanceProximite && 
				!(this.joueur.distancePlusProcheEquipier(this.joueur.getBalle()) < this.distanceProximite)) ||
				this.terrain.distance(this.joueur.getBalle(), this.joueur.getButAdverse()) < distanceDefensive) {
			this.verb("défense");
			this.reflechirDefense();
		} else {
			this.verb("attaque");
			this.reflechirPositionnement();
		}
	}
	
	public void reflechirPositionnement() {
		//on regarde que l'on soit au point de démarcation
		if (distance(this.getPlusProchePointDemarcation()) < this.toleranceDemarcation) {
			//Si on est mal positionné on se démarque
			if (!this.trajectoireOk(this.joueur.getAttaquant(), this.joueur.getJoueur())) {
				this.seDemarquer();
			} else {
				//sinon on s'oriente vers l'attaquant
				//tourner(this.angle(this.joueur.getAttaquant()));
				//la balle plutôt... ses coordonnées sont transmises entre les joueurs
				this.tourner(this.angle(this.joueur.getBalle()));
			}
		} else {
			//sinon on y va
			/**
			 * Ici le point le plus proche est souvent hésitant entre deux 
			 * et le robot se fige comme un con, il faudrait réaliser une classe 
			 * permettant une persistence des valeurs et qui les renouvellent régulièrement
			 */
			this.allerA(this.getPlusProchePointDemarcation(), false);
		}
	}
	
	public void reflechirDefense() {
		if (this.terrain.distance(this.adversairePlusProcheBalle(), this.joueur.getBalle()) > this.distanceAttaquantPossedeBalle) {
			//le joueur adverse n'a pas la balle, on le shoot pour le ralentir
			this.allerA(this.adversairePlusProcheBalle(), false);
		} else {
			//on se positionne entre l'attaquant adverse et le passeur le plus probable
			Vector relais = this.getRelaisAdverse(this.adversairePlusProcheBalle());
			Vector target = this.terrain.milieu(this.adversairePlusProcheBalle(), relais);
			if (this.distance(target) > distancePositionDefense) {
				this.allerA(target, false);
			} else {
				this.tourner(this.angle(this.joueur.getBalle()));
			}
		}
	}
	
	public Vector getRelaisAdverse(Vector attaquant) {
		Vector[] Adversaires = joueur.getAdversaires();
		double distance = -1;
		Vector adversaire = new Vector();
		
		for (int i=0;i<Adversaires.length;i++) {
			if ( ! Adversaires[i].equals(attaquant) ) {
				if (terrain.distance(Adversaires[i], joueur.getButAdverse()) < distance || distance == -1) {
					distance = terrain.distance(Adversaires[i], joueur.getButAdverse());
					adversaire = Adversaires[i];
				}
			}
		}
		return adversaire;
	}

	
	public void seDemarquer() {
		this.allerA(this.getPlusEloignePointDemarcation(), false);
	}
	
	
	public void verb(String str) {
		//System.out.println(this.joueur.title+" : "+str);
	}

	/*private Vector noeudPlusProche(Vector cible) {
		
		Vector[] noeuds = joueur.getNoeuds();
		// pour l'instant le noeud le plus proche est le premier
		Vector plusProche = noeuds[0];
		double distanceMin = terrain.distance(plusProche,cible);
		
		// pour tous les autres noeuds
		for (int i = 1; i < noeuds.length; i++) {
			// s'il sont plus proche que le dernier trouvé
			if (distance(noeuds[i]) < distanceMin) {
				// on enregistre la distance comme etant la plus petite
				distanceMin = terrain.distance(noeuds[i],cible);
				// on enregistre le noeud le plus proche
				plusProche = noeuds[i];
			}
		}
				
		return plusProche;
	}*/
	
	/**
	 * @return les points de démarcation par rapport au but
	 */
	private Vector[] getPointsDemarcation() {
		Vector[] ret = new Vector[2];
		
		Complex but = this.terrain.warp(this.joueur.getButEquipe()), attaquant = this.terrain.warp(this.joueur.getAttaquant());
		
		//Le vecteur orthogonale à celui entre le but et l'attaquant
		Complex orthVector = new Complex(- (but.y - attaquant.y) / (but.x - attaquant.x),1); /*immutable*/
		//orthVector.y = 1;
		//orthVector.x = - (but.y - attaquant.y) / (but.x - attaquant.x);
		
		double norm = Math.sqrt(orthVector.y * orthVector.y + orthVector.x * orthVector.x); 
		
		//orthVector.y /= norm;
		//orthVector.x /= norm;
		orthVector = orthVector.mul(1/norm);
		
		Complex point1 = new Complex(but.x + orthVector.x * this.distanceDemarcationBut, but.y + orthVector.y * this.distanceDemarcationBut);
		Complex point2 = new Complex(but.x - orthVector.x * this.distanceDemarcationBut, but.y - orthVector.y * this.distanceDemarcationBut);
		//point1.x = but.x + orthVector.x * this.distanceDemarcationBut;
		//point1.y = but.y + orthVector.y * this.distanceDemarcationBut;
		//point2.x = but.x - orthVector.x * this.distanceDemarcationBut;
		//point2.y = but.y - orthVector.y * this.distanceDemarcationBut;
		
		ret[0] = this.terrain.warp(point1);
		ret[1] = this.terrain.warp(point2);
		
		return ret;
	}
	
	/**
	 * @return le point de démarcation le plus proche
	 */
	private Vector getPlusProchePointDemarcation() {
		Vector[] points = this.getPointsDemarcation();
		
		//test : renvoyons tout le temps la même valeur
		return points[0];
		
		/*
		if (this.joueur.distance(points[0]) < this.joueur.distance(points[1])) {
			return points[0];
		}
		return points[1];*/
	}
	
	/**
	 * @return le point de démarcation le plus éloigné
	 */
	private Vector getPlusEloignePointDemarcation() {
		Vector[] points = this.getPointsDemarcation();
		
		return points[1];
		/*if (this.joueur.distance(points[0]) > this.joueur.distance(points[1])) {
			return points[0];
		}
		return points[1];*/
	}
	
}
