/**
 * 
 */
package hoverball.team.TribalQueens_;

import hoverball.Puck;
import hoverball.Unit;
import hoverball.debug.Circle;
import hoverball.debug.Text;
import hoverball.math.Matrix;
import hoverball.math.Vector;

/**
 * @author detantxavi
 * 
 */
public class TribalQueen extends Unit {

	private Environnement terrain;

	/***********************************************************
	 * Les éléments du jeu
	 ***********************************************************/

	/**
	 * Les 6 noeuds du terrain
	 */
	private Puck noeuds[] = new Puck[6];

	/**
	 * La balle du jeu ( a pousser vers le but adverse )
	 */
	private Puck balle;

	/**
	 * Le but de l'équipe ( à défendre )
	 */
	private Puck butEquipe;

	/**
	 * Le but de l'adversaire
	 */
	private Puck butAdverse;

	/**
	 * La taille d'une équipe
	 */
	private final int tailleEquipe = 3;

	/**
	 * Les equipiers du joueur y compris le joueur lui même
	 */
	private Puck equipiers[] = new Puck[tailleEquipe];

	/**
	 * Vrai si le ieme equipier est desactivé
	 */
	private boolean equipiersDesactive[] = new boolean[tailleEquipe];

	/**
	 * vrai si le role a été donnée au ieme joueur
	 */
	private boolean roleDonne[] = new boolean[tailleEquipe];

	/**
	 * Les adversaires du joueur.
	 */
	private Puck adversaires[] = new Puck[tailleEquipe];

	/**
	 * vrai si on a déjà tout vu
	 */
	private boolean toutVu;

	/**
	 * Le role du joueur
	 */
	private Role role;

	/**
	 * La puissance minimale des moteurs.
	 */
	private double puissanceMin;

	/**
	 * La puissance maximale des moteurs.
	 */
	private double puissanceMax;

	/**
	 * La charge minimale de l'energie.
	 */
	private double chargeMin;

	/**
	 * La charge maximale de l'energie.
	 */
	private double chargeMax;

	/**
	 * Le facteur de cout de charge
	 */
	private double coutCharge;

	/**
	 * La fréquence de jeu
	 */
	private double frequence;

	/**
	 * @param arg0
	 *            Le nom de l'équipe
	 * @param arg1
	 *            Le nom du joueur
	 * @param arg2
	 *            La couleur du joueur
	 */
	public TribalQueen(String arg0, String arg1, int arg2) {
		super(arg0, arg1, arg2);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see hoverball.Unit#loop()
	 */
	protected void loop() {
		Puck[] pucksTmp;
		Puck selfTmp;
		// on inisialise tout
		initialiser();

		// temps qu'on joue
		while (look()) {

			// par defaut, tous les equipiers sont activés
			for (int i = 0; i < equipiersDesactive.length; i++) {
				equipiersDesactive[i] = false;
			}
			
			// si j'ai déjà tout vu de moi même
			if (toutVu) {
				// on regarde via les equipiers
				for (int i = 0; i < pucks.length; i++) {
					if (pucks[i].equals(UNIT) && pucks[i].t == self.t
							&& !pucks[i].id.equals(self.id)) {
						// on se met à la place de l'equipier
						pucksTmp = pucks;
						selfTmp = self;
						// on recupère sa vue
						pucks = Communication.pucksFromString(pucks[i].message);
						// on verifie qu'il a bien fournit toutes les infos
						if (pucks.length == 5) {
							// on se met à sa place
							self = pucks[4];
							// on regarde autour de nous
							regarder(pucks);
						}
						// on se remet à sa place
						pucks = pucksTmp;
						self = selfTmp;
					}
				}
			}

			// on regarde autour de nous
			regarder(pucks);

			// si on a tout vu
			if (toutVu()) {
				tag(getJoueur(), title + "\nEnergie : " + format(energy) + "\n");
				
				// si on a pas de role ou si notre role n'est pas en situation critique
				if(role == null || !role.isCritique()) {
				// on choisi notre role
				choisirRole();
				}
				// on reflechis
				role.reflechir();

				action(role.getPolarisation(), role.getMoteurGauche(), role
						.getMoteurDroit(), Communication
						.pucksToString(getPuckToCommunicate()));
			} else {
				// si on a pas tout vu, on tourne
				action(0, puissanceMax, puissanceMin, Communication
						.pucksToString(getPuckToCommunicate()));
			}
		}
	}

	/**
	 * Initialise toutes les données
	 */
	private void initialiser() {
		// on initialise l'environnement de jeu
		terrain = new Environnement(option("world.radius"),
				option("ball.radius"));
		chargeMin = option("unit.charge.min");
		chargeMax = option("unit.charge.max");
		puissanceMin = option("unit.engine.min");
		puissanceMax = option("unit.engine.max");
		coutCharge = option("unit.charge.cost");
		frequence = 1.0 / 50;

		// pour l'instant on a rien vu
		balle = null;
		butEquipe = null;
		butAdverse = null;
		for (int i = 0; i < tailleEquipe; i++) {
			adversaires[i] = null;
			equipiers[i] = null;
		}
		toutVu = false;
	}

	/**
	 * Récupère toutes les informations disponibles autour de soit.
	 * 
	 * @param pucks
	 *            La liste des pucks vus par le joueur.
	 */
	private void regarder(Puck[] pucks) {

		// on met à jour la mémoire du joueur
		mettreAJourMemoire();

		// pour tous les pucks trouvés
		for (int i = 0; i < pucks.length; i++) {
			// si ils sont bien connus alors on les analyse
			if (connu(pucks[i])) {
				// si c'est un palet
				if (pucks[i].equals(BALL)) {
					// on l'enregistre
					paletTrouve(pucks[i]);
				} else if (pucks[i].equals(UNIT)) {
					// si c'est une unité
					uniteTrouve(pucks[i]);
				} else if (pucks[i].equals(NODE)) {
					// si c'est un noeud
					noeuds[pucks[i].n - 1] = new Puck(pucks[i]);
				}
			}
		}

	}

	/**
	 * Met à jour les donnée précédement enregistrés en fonction d'un noeud de
	 * repers .
	 */
	private void mettreAJourMemoire() {
		Matrix revision = null; // la matrice permettant de mettre à jour les
		// positions
		Puck ref;
		// on cherche un noeud de repère afin de trouver la matrice de mise à
		// jour
		for (int i = 0; i < noeuds.length; i++) {

			// on continue a chercher temps qu'on a pas de matrice ni de point
			// de repère
			if ((revision == null) && (noeuds[i] != null)) {
				try {
					ref = puck(NODE, 0, i + 1);
				} catch (NullPointerException e) {
					ref = null;
				}
				// si on connait le ieme node
				if (connu(ref)) {
					// on récupère alors notre matrice
					revision = Matrix.inv(noeuds[i].X).mul(ref.X);

					// on met à jour les coordonnées de la balle
					miseAJourCoordonnee(balle, revision);

					// on met à jour les coordonnées du but de l'équipe
					miseAJourCoordonnee(butEquipe, revision);

					// on met à jour les coordonnées du but adverse
					miseAJourCoordonnee(butAdverse, revision);

					// on met à jour les équipes
					for (int j = 0; j < tailleEquipe; j++) {
						miseAJourCoordonnee(equipiers[j], revision);
						miseAJourCoordonnee(adversaires[j], revision);
					}

					// on met à jour les noeuds
					for (int j = 0; j < noeuds.length; j++) {
						miseAJourCoordonnee(noeuds[j], revision);
					}

					// pas besoin de continuer la boucle qui ne fera plus rien
					break;
				}

			}

		}

	}

	/**
	 * Dis si un puck est connu ou non.
	 * 
	 * @param puck
	 *            Le puck recherché.
	 * @return Vrai si le puck est connu et faux sinon.
	 */
	public static boolean connu(Puck puck) {
		// on connait le puck s'il n'est pas null et s'il a des coordonnée
		return ((puck != null) && (puck.X != null));
	}

	/**
	 * Met à jour les coordonnées d'un puck en fonction d'une matrice de
	 * révision.
	 * 
	 * @param puck
	 *            Le puck à mettre à jour.
	 * @param revision
	 *            La matrice de révision.
	 */
	private void miseAJourCoordonnee(Puck puck, Matrix revision) {
		// si le puck est connu
		if (connu(puck)) {
			// alors on met à jour ses coordonée
			puck.X = puck.X.mul(revision); /*immutable*/
		}
	}

	/**
	 * Appellé quand on a trouvé un palet afin de savoir de quel palet il s'agit
	 * et de l'enregistrer
	 * 
	 * @param puck
	 *            Le palet trouvé
	 */
	private void paletTrouve(Puck puck) {
		// si le palet est neutre alors il s'agit de la balle
		if (puck.t == 0) {
			// on enregistre alors sa position
			balle = new Puck(puck);
		} else if (puck.t == self.t) {
			// sinon si le palet est de l'équipe du joueur alors se sont ses but
			// on enregistre leur position
			butEquipe = new Puck(puck);
		} else {
			// si on arrive ici c'est que le palet appartient à l'adversaire, on
			// enregistre donc la position de leur cage.
			butAdverse = new Puck(puck);
		}
	}

	/**
	 * Appellé quand on a trouvé une unité afin de savoir à quelle équipe elle
	 * appartient et de l'enregistrer
	 * 
	 * @param puck
	 *            L'unité trouvée
	 */
	private void uniteTrouve(Puck puck) {
		// si l'unité est dans la même équipe
		if (puck.t == self.t) {
			// on enregistre le nieme equipier
			equipiers[puck.n - 1] = new Puck(puck);
		} else if (puck.t == 9) {
			// si le puck est de l'equipe moins 1 alors c'est une equipier desactivé
			equipiersDesactive[puck.n - 1] = true;
			equipiers[puck.n - 1] = new Puck(puck);
			//equipiers[puck.n - 1] = new Puck(puck.what, self.t, puck.n,puck.X,0,0,"");
		} else {
			// si on arrive ici c'est que l'unité est dans l'équipe adverse
			// on enregistre le nieme adversaire
			adversaires[puck.n - 1] = new Puck(puck);
		}
	}

	/**
	 * @return Vrai si tout a été vu au moins une fois
	 */
	private boolean toutVu() {
		// si on a déjà tout vu, pas besoin de verifier
		if (!toutVu) {
			// pour l'instant on a tout vu
			toutVu = true;

			// on doit connaitre la balle, le but adverse et le but de l'équipe
			toutVu = toutVu && connu(balle) && connu(butAdverse)
					&& connu(butEquipe);

			// on doit connaitre tous les adversaires et tous les coequipiers
			for (int i = 0; i < adversaires.length; i++) {
				toutVu = toutVu && connu(adversaires[i]) && connu(equipiers[i]);
			}

			// on doit aussi avoir vu tous les noeuds
			for (int i = 0; i < noeuds.length; i++) {
				toutVu = toutVu && connu(noeuds[i]);
			}

		}
		// on dis si on a tout vu
		return toutVu;
	}

	/**
	 * Tag un puck pour le debugage
	 * 
	 * @param p
	 *            Le puck a tagger
	 * @param s
	 *            Le nom du tag
	 */
	public void tag(Vector point, String s) {
		s = s + "\nDistance :" + format(terrain.distance(point, puckToVector(self)))
				 + "\nAngle :" + format(terrain.angle(point));

		debug(new Circle(point, 0.06), getColor());
		debug(new Text(point, s, 2, 0), getColor());
	}

	/**
	 * @return La couleur de debug en fonction du joueur
	 */
	private int getColor() {
		if (this.title.equals("Peach")) {
			return (0xFF00AA);
		} else if (this.title.equals("Zelda")) {
			return (0x00FF00);
		}
		return 0xFFAA00;
	}

	/**
	 * Choisi le role du joueur en fonction de l'environnement
	 */
	private void choisirRole() {
		// tag(getBalle(), "Balle");
		// tag(getButEquipe(), "But");
		// tag(getButAdverse(), "But Adverse");
		// on n'affecte aucun role pour l'instant
		for (int i = 0; i < roleDonne.length; i++) {
			roleDonne[i] = false;
			// tag(getEquipiers()[i], "Coop" + i);
		}

		// si on est le plus proche de la balle
		if (plusProche(getBalle())) {
			// System.out.println(this.title+" Attaque");
			// alors on attaque
			role = new Attaquant(this, terrain);
		} else if ( (terrain.distance(getBalle(), getButAdverse()) < 40 && plusProche(terrain.milieu(getButAdverse(),getBalle())))
		||		(terrain.distance(getBalle(), getButAdverse()) >= 40 && plusProche(getButAdverse()))
		) {
			// System.out.println(this.title+" Defend");
			// sinon si on est les plus proche du but adverse alors on defends sauf si qq'un est déjà mieux placé
			role = new Defenseur(this, terrain);
		} else {
			// System.out.println(this.title+" Relais");
			// sinon alors on est en relais
			role = new Relais(this, terrain);
		}

		/*
		 * // si on est le plus proche de la balle if
		 * (this.title.equals("Peach")) { // alors on est en attaque role = new
		 * Attaquant(this, terrain); } else if (this.title.equals("Zelda")) { //
		 * alors on defends role = new Defenseur(this, terrain); } else { //
		 * alors on est en relais role = new Relais(this, terrain); }
		 */

	}

	/**
	 * @param point
	 * @return Vrai si on est le plus proche du point
	 */
	private boolean plusProche(Vector point) {
		Vector[] equipiers = getEquipiers();
		int joueur;

		// on cherche le premier joueur qui n'a pas de role
		for (joueur = 0; joueur < equipiers.length; joueur++) {
			// si l'equipier est le joueur
			if (!roleDonne[joueur]) {
				// alors on a trouver le joueur
				break;
			}
		}

		// pour tous les equipiers
		for (int i = 1; i < equipiers.length; i++) {
			// si le joueur n'a pas déjà un role et n'est pas desactive et s'il est plus proche du but
			// que le precedent
			if (!roleDonne[i] && !equipiersDesactive[i]
					&& terrain.distance(equipiers[i], point) < terrain
							.distance(equipiers[joueur], point)) {
				// on a un joueur plus proche
				joueur = i;
			}
		}
		// on affecte un role au joueur
		roleDonne[joueur] = true;

		// on est le plus proche si le joueur trouvé est moi meme
		return equipiers[joueur].equals(getJoueur());
	}

	/**
	 * Transforme un puck en vecteur
	 * 
	 * @param puck
	 * @return
	 */
	private Vector puckToVector(Puck puck) {
		return puck.X.c;
	}

	/**
	 * @return the balle
	 */
	public Vector getBalle() {
		return puckToVector(balle);
	}

	/**
	 * @return the butEquipe
	 */
	public Vector getButEquipe() {
		return puckToVector(butEquipe);
	}

	/**
	 * @return the butAdverse
	 */
	public Vector getButAdverse() {
		return puckToVector(butAdverse);
	}

	/**
	 * @return the equipiers
	 */
	public Vector[] getEquipiers() {
		// on creer le tableau des equipiers
		Vector equipiersVector[] = new Vector[equipiers.length];

		// on transforme chaque équipier
		for (int i = 0; i < equipiers.length; i++) {
			equipiersVector[i] = puckToVector(equipiers[i]);
		}

		return equipiersVector;
	}

	/**
	 * @return the adversaires
	 */
	public Vector[] getAdversaires() {
		// on creer le tableau des adversaires
		Vector adversairesVector[] = new Vector[adversaires.length];

		// on transforme chaque adversaire
		for (int i = 0; i < adversaires.length; i++) {
			adversairesVector[i] = puckToVector(adversaires[i]);
		}

		return adversairesVector;
	}

	/**
	 * @return le joueur
	 */
	public Vector getJoueur() {
		return puckToVector(self);
	}

	/**
	 * @return l'energie du joueur
	 */
	public double getEnergy() {
		return energy;
	}

	/**
	 * @return the puissanceMin
	 */
	public double getPuissanceMin() {
		return puissanceMin;
	}

	/**
	 * @return the puissanceMax
	 */
	public double getPuissanceMax() {
		return puissanceMax;
	}

	/**
	 * @return the chargeMin
	 */
	public double getChargeMin() {
		return chargeMin;
	}

	/**
	 * @return the chargeMax
	 */
	public double getChargeMax() {
		return chargeMax;
	}

	/**
	 * @return the coutCharge
	 */
	public double getCoutCharge() {
		return coutCharge;
	}

	/**
	 * @return the frequence
	 */
	public double getFrequence() {
		return frequence;
	}

	/**
	 * @return the noeuds
	 */
	public Vector[] getNoeuds() {
		Vector[] res = new Vector[noeuds.length];

		// pour tous les noeuds
		for (int i = 0; i < noeuds.length; i++) {
			// on les transforme en vecteur
			res[i] = puckToVector(noeuds[i]);
		}

		return res;
	}

	/**
	 * @return La liste des pucks à retourner
	 */
	private Puck[] getPuckToCommunicate() {
		Puck[] envoi = new Puck[5];

		// on envoie un noeud de repère
		for (int i = 0; i < noeuds.length; i++) {
			if (connu(noeuds[i])) {
				envoi[0] = noeuds[i];
			}
		}

		// on envoie la balle
		envoi[1] = balle;

		// les buts
		envoi[2] = butEquipe;
		envoi[3] = butAdverse;

		// moi meme
		envoi[4] = self;

		// si je suis desactivé
		if (this.energy <= 0.2) {
			envoi[4].message = "desactive";
		} else {
			envoi[4].message = "ok";
		}
		
		return envoi;
	}

	public double distancePlusProcheAdversaire(Vector target) {
		double ndist, ret = -1;
		Vector[] adversaires = this.getAdversaires();
		for (int i = 0; i < adversaires.length; i++) {
			ndist = this.terrain.distance(target, adversaires[i]);
			if ((ret == -1) || (ndist < ret)) {
				ret = ndist;
			}
		}
		return ret;
	}

	public double distancePlusProcheEquipier(Vector target) {
		double ndist, ret = -1;
		Vector[] equipiers = this.getEquipiers();
		for (int i = 0; i < equipiers.length; i++) {
			ndist = this.terrain.distance(target, equipiers[i]);
			if ((ret == -1) || (ndist < ret)) {
				ret = ndist;
			}
		}
		return ret;
	}

	/**
	 * Renvoie le vector de l'attaquant de l'équipe (le plus proche de la balle)
	 * 
	 * @return
	 */
	public Vector getAttaquant() {
		double distance = -1;
		int min = -1;
		Vector[] equipiers = this.getEquipiers();
		for (int i = 0; i < 1; i++) {
			double ndist = this.terrain.distance(equipiers[i], this.getBalle());
			if (ndist < distance || min == -1) {
				distance = ndist;
				min = i;
			}
		}
		return equipiers[min];
	}

	/**
	 * @return the equipiersDesactive
	 */
	public boolean[] getEquipiersDesactive() {
		return equipiersDesactive;
	}
	
}
