package hoverball.team.TrioGaulois_;

import hoverball.*;
import hoverball.debug.*;
import hoverball.math.*;



public class TrioGaulois extends Team
{
  // ---- Application

  public void close () { super.close(); }

  public static void main (String[] args)
  {
    if (args.length == 0) new TrioGaulois().show();
    else if (args.length == 1) new TrioGaulois(args[0]).show();
    else new TrioGaulois(args[0],Integer.parseInt(args[1],16)).show();
  }

  // ---- Constructor

  public TrioGaulois () { this(null); }
  public TrioGaulois (String team) { this(team,0xEEAAAA); }
  public TrioGaulois (int color) { this(null,color); }
  public TrioGaulois (String team, int color)
  {
    super((team == null || team == "")? "Trio Gaulois" : team);

    add(new Gaulois(null,"Marc",color,Gaulois.MARC));
    add(new Gaulois(null,"Cedric",color,Gaulois.CEDRIC));
    add(new Gaulois(null,"Breton", color,Gaulois.BRETON));
  }
}



class Gaulois extends Unit
{
  private final String name;                // mein Name
  private final int identity;               // meine Nummer (MOONDANCER = 0, MADDRAGON = 1, ANGELFIRE = 2)
  private int nbOpponents = 0;        // zaehlt entdeckte Gegnerunits, 3 maximal (fuer regulaeres Spiel)
  private boolean pucksLocated = false;    // ich kann erst losspielen, wenn ich alle Pucks geortet habe
  // aktuellen Daten
  private Puck ball = null;               // der Ball
  private Puck goal = null;               // das eigene Tor
  private Puck flop = null;               // das gegnerische Tor
  private Puck friend[] = new Puck[3];    // mein Team (ich eingeschlossen)
  private Puck opponent[] = new Puck[3];  // Gegnerteam
  private Puck node[] = new Puck[6];      // die Knoten
  private Puck defenseur = null;
  private Puck attaquant = null;
  private Puck porteur = null;
  private Puck visavis1 = null;
  private Puck visavis2 = null;
  private Puck visavis3 = null;

  // die Knoten, die ich das letzte Mal gesehen habe
  private Puck node_[] = new Puck[6];      // die alten Knoten

  private double P = 0; // Aktion: Polarisation
  private double L = 0; // Aktion: linker Motor
  private double R = 0; // Aktion: rechter Motor

  private final double DANGERRADIUS = 20;   // Abstand zum Teamball von Moondancer als Torhueter und von Mad Dragon / Angelfire auf Lauerstellung
  private final double AVOIDRADIUS_GOAL = 15;   // Ab dieser Distanz muesses Teamballs umfahren werden
  private final double AVOIDRADIUS_GEGNER = 20; //Contourner l'adversaire
  private final double DISTANCE_POSSESSION = 4; //Un joueur a la balle si il est a moins de la valeur

  final static int MARC = 0;
  final static int CEDRIC = 1;
  final static int BRETON = 2;

  String reference = ""; // der Referenzknoten

  // ---- Constructor ---------------------------------------------------------

  Gaulois (String team, String name, int color, int identity) {
    super(team,name,color);
    this.name = name;
    this.identity = identity;
  }

  // ---- Loop ----------------------------------------------------------------

  private Sphere sphere     = null;         // die Kugeloberfl?che
  private double sphere_rad = 0;            // Kugelradius
  private double charge_max = 0;            // max. positive Polarisation
  private double charge_min = 0;            // max. positive Polarisation
  private double engine_min = 0;            // min negativer Motorenwert
  private double engine_max = 0;            // max positiver Motorenwert
  private double distanceBallUnit = 0;      // minimaler Abstand Ball-Unit
  private double radius = 0;                // eigener Radius

  public void loop () {
    sphere_rad = option("world.radius");
    sphere = new Sphere(sphere_rad);
    charge_min = option("unit.charge.min");
    charge_max = option("unit.charge.max");
    engine_min = option("unit.engine.min");
    engine_max = option("unit.engine.max");
    radius = option("unit.radius");
    distanceBallUnit = radius + option("ball.radius");

    while (look())  // loop structure:  while (look()) { ...; action(...); }
    	    {
      P = 0;  // Aktionen initialisieren
      L = 0;
      R = 0;
      String actionMessage = "";

      // tempor?re Variablen zur Berechnung
      Complex c0 = null;
      Complex c1 = null;
      Complex c2 = null;
      Complex c3 = null;
      Complex c3_ = null;
      Complex c4 = null;
      Complex c4_ = null;
      Vector v1 = null;
      Vector v2 = null;
      Vector v3 = null;

      double d1,d2,d3;
      double dist1,dist2,dist3;
      double pressing1,pressing2,pressing3;


      detect(pucks);

      if (!pucksLocated) { // noch nicht alles gesehen, dreh Dich
    	rotate();
        actionMessage = "detect";
      }
      else {

        //Identification du defenseur

        switch (identity) {
          case MARC: // Marc ist der Scorer
              if (see(flop))
        	  {
                  //Phase offensive
                    d1 = (opponent[0] == null)? 0 : distance(opponent[0].X.c,goal.X.c);
                    d2 = (opponent[1] == null)? 0 : distance(opponent[1].X.c,goal.X.c);
                    d3 = (opponent[2] == null)? 0 : distance(opponent[2].X.c,goal.X.c);

                    if (d1<d2 && d1<d3){
                        defenseur = opponent[0];
                    }
                    if(d2<d1 && d2<d3){
                        defenseur = opponent[1];
                    }
                    if(d3<d1 && d3<d2){
                        defenseur = opponent[2];
                    }

                    /*//Phase defensive
                    dist1 = distance(opponent[0].X.c,ball.X.c);
                    dist2 = distance(opponent[1].X.c,ball.X.c);
                    dist3 = distance(opponent[2].X.c,ball.X.c);

                    if (dist1<dist2 && dist1<dist3){
                        porteur = opponent[0];
                    }
                    if(dist2<dist1 && dist2<dist3){
                        porteur = opponent[1];
                    }
                    if(dist3<dist1 && dist3<dist2){
                        porteur = opponent[2];
                    }

                    pressing1 = distance(friend[0].X.c,porteur.X.c);
                    pressing2 = distance(friend[1].X.c,porteur.X.c);
                    pressing3 = distance(friend[2].X.c,porteur.X.c);

                    if (pressing1<pressing2 && pressing1<pressing3){
                        visavis1 = opponent[0];
                    }
                    if(pressing2<pressing1 && pressing2<pressing3){
                        visavis2 = opponent[1];
                    }
                    if(pressing3<pressing1 && pressing3<pressing2){
                        visavis3 = opponent[2];
                    }*/




//System.out.println(nbOpponents);
        		  c1 = warp(flop.X);
        		  c2 = warp(ball.X);
        		  c3 = Complex.sub(c2,c1).mul(Complex.expi(Math.PI/32));
        		  c4 = Complex.add(c1,Complex.mul(c3,DANGERRADIUS/c3.abs()*1.3));
        		  v1 = sphere.warp(c4);
                          if (distance(ball.X.c,friend[1].X.c)<=DISTANCE_POSSESSION+2){
                                  //turnTo(friend[1].X.c);
                                  goTo(defenseur.X.c);
                                  debug (new Line(new Vector(0,0,0),defenseur.X.c));
                                  actionMessage = "va gener";
                          }
                          else if (distance(ball.X.c,friend[2].X.c)<=DISTANCE_POSSESSION+2){
                                  turnTo(friend[2].X.c);
                                  actionMessage = "reception";
                          }
                          /*else if (distance(ball.X.c,opponent[0].X.c)<=DISTANCE_POSSESSION+2 || distance(ball.X.c,opponent[1].X.c)<=DISTANCE_POSSESSION+2 || distance(ball.X.c,opponent[2].X.c)<=DISTANCE_POSSESSION+2){
                                  goTo(v1);
                                  actionMessage = "replacement defensif";
                          }*/
                          else if ((distanceTo(ball)<5*distanceBallUnit) || sphere.diff(v1,self.X.c)>0.2*radius) {
        			  goToGoal(ball.X.c,goal.X.c,0.01);
                                  actionMessage = "goToGoal";
                                  debug (new Line(new Vector(0,0,0),goal.X.c));
                                  debug (new Line(new Vector(0,0,0),ball.X.c));
        		  }
                          else if(distance(self.X.c,flop.X.c)<=25){
                                  shoot(ball.X.c,goal.X.c,0.01);
                                  actionMessage = "conclusion";
                          }
        		  else if (sphere.diff(v1,self.X.c)>0.2*radius) { // stell Dich in die Verbindunglinie ball - flop
        			  goTo(v1);
        			  actionMessage = "pressing";
        		  }
        		  else { // dreh Dich und beobachte
        			  rotate();
        			  actionMessage = "rotate";
        		  }
        		  //debug (new Circle(goal.X.c,DANGERRADIUS/sphere_rad));
        		  //debug (new Line(new Vector(0,0,0),ball.X.c));
        	  }

              else { // dreh Dich und such den flop
        			  rotate();
        			  actionMessage = "search";
        	  }
          break;

          case CEDRIC: // 2eme attaquant
              if (see(flop))
        	  {
                    d1 = (opponent[0] == null)? 0 : distance(opponent[0].X.c,goal.X.c);
                    d2 = (opponent[1] == null)? 0 : distance(opponent[1].X.c,goal.X.c);
                    d3 = (opponent[2] == null)? 0 : distance(opponent[2].X.c,goal.X.c);

                    if (d1<d2 && d1<d3){
                        defenseur = opponent[0];
                    }
                    if(d2<d1 && d2<d3){
                        defenseur = opponent[1];
                    }
                    if(d3<d1 && d3<d2){
                        defenseur = opponent[2];
                    }

        		  c1 = warp(flop.X);
        		  c2 = warp(ball.X);
        		  c3 = Complex.sub(c2,c1).mul(Complex.expi(Math.PI/32));
        		  c4 = Complex.add(c1,Complex.mul(c3,DANGERRADIUS/c3.abs()*1.3));
        		  v1 = sphere.warp(c4);
                          if (distance(ball.X.c,friend[2].X.c)<=DISTANCE_POSSESSION+2){
                                  turnTo(friend[2].X.c);
                                  actionMessage = "reception";
                          }
                          else if (distance(ball.X.c,friend[0].X.c)<=DISTANCE_POSSESSION+2){
                                  //turnTo(friend[0].X.c);
                                  goTo(defenseur.X.c);
                                  actionMessage = "va gener";
                                  debug (new Line(new Vector(0,0,0),defenseur.X.c));
                          }
                          /*else if (distance(ball.X.c,opponent[0].X.c)<=DISTANCE_POSSESSION+1 || distance(ball.X.c,opponent[1].X.c)<=DISTANCE_POSSESSION+1 || distance(ball.X.c,opponent[2].X.c)<=DISTANCE_POSSESSION+1){
                                  goTo(v1);
                                  actionMessage = "replacement defensif";
                          }*/
                          else if ((distanceTo(ball)<5*distanceBallUnit) || sphere.diff(v1,self.X.c)>0.2*radius) {
        			  goToGoal(ball.X.c,goal.X.c,0.01);
                                  actionMessage = "goToGoal";
                                  debug (new Line(new Vector(0,0,0),goal.X.c));
                                  debug (new Line(new Vector(0,0,0),ball.X.c));
        		  }
                          else if(distance(self.X.c,flop.X.c)<=25){
                                  shoot(ball.X.c,goal.X.c,0.01);
                                  actionMessage = "conclusion";
                          }
        		  else if (sphere.diff(v1,self.X.c)>0.2*radius) { // stell Dich in die Verbindunglinie ball - flop
        			  goTo(v1);
        			  actionMessage = "pressing";
        		  }
        		  else { // dreh Dich und beobachte
        			  rotate();
        			  actionMessage = "rotate";
        		  }
        		  //debug (new Circle(goal.X.c,DANGERRADIUS/sphere_rad));
        		  //debug (new Line(new Vector(0,0,0),ball.X.c));
        	  }
        	  else { // dreh Dich und such den flop
        			  rotate();
        			  actionMessage = "search";
        	  }
          break;


          case BRETON: // Def & passeur
              if (see(flop))
        	  {

            	  d1 = (opponent[0] == null)? 0 : distance(opponent[0].X.c,flop.X.c);
            	  d2 = (opponent[1] == null)? 0 : distance(opponent[1].X.c,flop.X.c);
            	  d3 = (opponent[2] == null)? 0 : distance(opponent[2].X.c,flop.X.c);

                    if (d1<d2 && d1<d3){
                        attaquant = opponent[0];
                    }
                    if(d2<d1 && d2<d3){
                        attaquant = opponent[1];
                    }
                    if(d3<d1 && d3<d2){
                        attaquant = opponent[2];
                    }
        		  c0 = warp(goal.X);
        		  c1 = warp(flop.X);
        		  c2 = warp(ball.X);
        		  c3 = Complex.sub(c2,c1).mul(Complex.expi(Math.PI/32));
                          c3_ = Complex.sub(c2,c0).mul(Complex.expi(Math.PI/32));
        		  c4 = Complex.add(c1,Complex.mul(c3,DANGERRADIUS/c3.abs()*1));
                          c4_ = Complex.add(c1,Complex.mul(c3_,DANGERRADIUS/c3_.abs()*1.5));
        		  v1 = sphere.warp(c4);
                          v2 = sphere.warp(c1);
                          v3 = sphere.warp(c4_);

                          if (((distanceTo(ball)<15) || (distance(ball,flop)<DANGERRADIUS)) && distance(friend[0].X.c,ball.X.c)>DISTANCE_POSSESSION && distance(friend[1].X.c,ball.X.c)>DISTANCE_POSSESSION) { // Gefahr, weg mit dem Ball!
        			  passe(ball.X.c,friend[0].X.c,0.01);
        			  actionMessage = "passe";
                                  //shoot(ball.X.c,goal.X.c,0.01);
                                  //actionMessage = "conclusion";
        		  }
                          else if (attaquant != null && distance(attaquant.X.c,flop.X.c)<20){
                              P = charge_max;
                          }
        		  else if (sphere.diff(v1,self.X.c)>0.2*radius) { // stell Dich in die Verbindunglinie ball - flop
        			  goTo_Def(v1);
        			  actionMessage = "position";
                                  debug (new Line(new Vector(0,0,0),v1));
        		  }
        		  else { // dreh Dich und beobachte
        			  rotate();
        			  actionMessage = "rotate";
        		  }
        		  debug (new Circle(flop.X.c,DANGERRADIUS/sphere_rad));
        		  //debug (new Line(new Vector(0,0,0),v1));
        	  }
        	  else { // dreh Dich und such den flop
        			  rotate();
        			  actionMessage = "search";
        	  }
          break;

        }
      }
      action(P,L,R,""+identity); // abschicken der Aktionen
       // ----------------- Debugging ------------------------
      debug (new Text(null,name,1,0));
      debug (new Text(null,actionMessage,2,0));
      //debug (new Text(null,"reference: "+reference,3,0));
      // ------------------- Remember -----------------------
     } // look
  } // loop

  /*************************************************************************
   * function clear: Vector
   ************************************************************************/

  private void clear(Vector v) {
    if (distanceTo(v)<1.5* distanceBallUnit)
      P = charge_max;
    else {
      P = 0;
      goTo(v);
    }
  }

 /*************************************************************************
  * function shoot: Vector, Vector, double
  ************************************************************************/

   private void goToGoal(Vector object, Vector target, double angle) {
  // Schiesst ein Object in eine gewisse Richtung target,
  // wenn der Wingel angle erreicht ist.
    Complex oc = sphere.warp(object);
    Complex tc = sphere.warp(target);
    if ((distanceTo(object)>2*distanceBallUnit))
    {
        P = 0;
        goTo(object);
    }
    else if(distance(self.X.c,goal.X.c)<=25){
                                  shoot(ball.X.c,goal.X.c,0.01);
                                  //System.out.println("Position de tir");
    }
    else if ((Math.abs(oc.arg()-tc.arg())>angle))
    {
    	boolean opponentNear = false;
    	for (int i=0; i<nbOpponents; i++) if (distance(object,opponent[i].X.c)<0*distanceBallUnit) opponentNear = true;
    	if (opponentNear) P = charge_max; // clear is there is an opponent
    		else P = charge_min/100.0;
        goTo(target);
    }
    else
    {
    	P = charge_max;
    }
  }





  private void shoot(Vector object, Vector target, double angle) {
  // Schiesst ein Object in eine gewisse Richtung target,
  // wenn der Wingel angle erreicht ist.
    Complex oc = sphere.warp(object);
    Complex tc = sphere.warp(target);
    if ((distanceTo(object)>2*distanceBallUnit))
    {
        P = 0;
        goTo(object);
    }
    else if ((Math.abs(oc.arg()-tc.arg())>angle))
    {
    	boolean opponentNear = false;
    	for (int i=0; i<nbOpponents; i++) if (distance(object,opponent[i].X.c)<-3*distanceBallUnit) opponentNear = true;
    	if (opponentNear) P = charge_max; // clear is there is an opponent
    		else P = charge_min/100.0;
        turnTo(target);
    }
    else
    {
    	P = charge_max;
    }
  }

  private void passe(Vector object, Vector target, double angle) {
  // Schiesst ein Object in eine gewisse Richtung target,
  // wenn der Wingel angle erreicht ist.
    Complex oc = sphere.warp(object);
    Complex tc = sphere.warp(target);
    if ((distanceTo(object)>2*distanceBallUnit))
    {
        P = 0;
        goTo(object);
    }
    else if(distance(self.X.c,goal.X.c)<=25){
                                  shoot(ball.X.c,goal.X.c,0.01);
                                  //System.out.println("Position de tir");
    }
    else if ((Math.abs(oc.arg()-tc.arg())>angle))
    {
    	boolean opponentNear = false;
    	for (int i=0; i<nbOpponents; i++) if (distance(object,opponent[i].X.c)<5) opponentNear = true;
    	if (opponentNear) P = charge_max; // clear is there is an opponent
    		else P = charge_min/100.0;
        turnTo(target);
    }
    else
    {
    	P = distance(object,target)/200.0;
        //System.out.println("distance : " + distance(object,target) + " ||| charge : " + P);

    }
  }

  /**************************************************************************
   * function avoid: Vector
   *************************************************************************/

  private Vector avoid (Vector v) {
  // Man muss goal und flop vermeiden, wenn man auf v zuf?hrt
	Vector x = avoid(v,goal,AVOIDRADIUS_GOAL);
        if ((see(flop)))
        {
            x = avoid(x,flop,AVOIDRADIUS_GOAL);
        }
        if ((see(opponent[1])))
        {
            x = avoid(x,opponent[1],AVOIDRADIUS_GEGNER);
        }
        if ((see(opponent[2])))
        {
            x = avoid(x,opponent[2],AVOIDRADIUS_GEGNER);
        }
        if ((see(opponent[0])))
        {
            x = avoid(x,opponent[0],AVOIDRADIUS_GEGNER);
        }

       return x;
  }

  private Vector avoid(Vector v, Puck p, double AVOIDRADIUS) {
  // Vermeidet den Puck p
    if (p!=null) if (distanceTo(p)<AVOIDRADIUS) {
    // Hindernis umfahren
      Complex c1 = warp(p.X);
      Complex c2 = sphere.warp(v);
      double ang = triangle(new Complex(),c1,c2);
      if (Math.abs(ang)>Math.PI/2)
        return sphere.warp(Complex.add(Complex.sub(c2,c1).mul(Complex.expi(-ang/2)),c1));
    }
    return v;
  }

  /**************************************************************************
   * function goTo: Vector
   *************************************************************************/

  private void goTo(Vector v) {
  // der Spieler f?hrt zum Vector v und vermeidet dabei goal und flop

    Vector v_ = avoid(v);
    double angle = angle(v_)/(Math.PI*1.8); //angle in [-1,1]
    if (Math.abs(angle)>0.05) turnTo(v_); // Ziel nicht vor mir -> dreh dich
    else { // geradeaus volle Power
      double d = distanceTo(v_);
  	  L = engine_max*d*d;
   	  R = engine_max*d*d;
     }
  }

   private void goTo_Def(Vector v) {
  // der Spieler f?hrt zum Vector v und vermeidet dabei goal und flop

    Vector v_ = avoid(v);
    double angle = angle(v_)/(Math.PI*1); //angle in [-1,1]
    if (Math.abs(angle)>0.05) turnTo(v_); // Ziel nicht vor mir -> dreh dich
    else { // geradeaus volle Power
      double d = distanceTo(v_);
  	  L = engine_max*d*d*d;
   	  R = engine_max*d*d*d;
     }
  }
  /**************************************************************************
   * function turnTo: Vector
   *************************************************************************/

  private void turnTo(Vector v) {
    double angle = angle(v)/Math.PI; //angle in [-1,1]
    L = -100*engine_max*angle;
    R = -L;
  }

  /**************************************************************************
   * function rotate: no argument
   *************************************************************************/
  private void rotate() {
      L = engine_max;
      R = engine_min;
  }

  // ************************ Weitere Hilfsfunktionen *****************************

  private Complex warp (Matrix M) {
  // 3D-Matrix -> 2D komplexer Vektor
    Vector v = sphere.warp(M);
    return new Complex(v.x,v.y);
  }

  private double angle(Vector v) {
  // berechnet den Winkel zum Vector v
    if (v!=null) return new Complex(v).arg(); else return 0;
  }

  private double triangle(Complex a,Complex v,Complex b) {
  // berechnet den Winkel zwischen Vector v und a und b, v am Winkel
    if ((v!=null) && (a!=null) && (b!=null)) {
      Complex va = Complex.sub(a,v);
      Complex vb = Complex.sub(b,v);
      double x = vb.arg()-va.arg();
      if (x>Math.PI) x-=2*Math.PI;
      else if (x<-Math.PI) x+=2*Math.PI;
      return x;
    }
    else return 2*Math.PI;
  }

  private double distance(Vector v1, Vector v2) {
  // berechnet die Distanz zwischen den Vektoren v1 und v2
    if ((v1!=null) && (v2!=null)) return sphere.diff(v1,v2); else return sphere.rad*7;
  }

  private double distance(Puck p1,Puck p2) {
  // berechnet die Distanz zwischen den Pucks p1 und p2
    if ((p1!=null) && (p2!=null)) return distance(p1.X.c,p2.X.c); else return sphere.rad*7;
  }

  private double distanceTo(Puck p) {
  // berechnet die Distanz zwischen mir und Puck p
    if (p!=null) return distance(self.X.c,p.X.c);
    else return sphere.rad*7;
  }

  private double distanceTo(Vector v) {
  // berechnet die Distanz zwischen mir und Vector v
    return distance(self.X.c,v);
  }

  private boolean see(Puck p)
  {
	  if (p==null) return false;
	  if (p.X==null) return false;
	  return true;
  }

  private void detect(Puck[] pucks) {
    // Sucht die Pucks aus dem Array pucks zusammen und speichert alles sch?n ordentlich
    // in den Variablen ball, goal, flop, friend, opponent, node

    // Korrigiere die bekannten Daten anhand eines Knotens
	Matrix m = null;
	for (int i=0; i<6; ++i)
		if ((m==null) && (node_[i]!=null)) if (see(puck(NODE,0,i+1))) {
			reference = "node"+(i+1);
			m = Matrix.inv(node_[i].X).mul(puck(NODE,0,i+1).X);
			break;
		}

    if (m!=null) // muesste eigentlich immer der Fall sein, wenn unit.vision = 0.66
    {
        if (see(ball)) ball.X = ball.X.mul(m); /*immutable*/
        if (see(goal)) goal.X = goal.X.mul(m); /*immutable*/
        if (see(flop)) flop.X = flop.X.mul(m); /*immutable*/
        for (int k=0; k<3; ++k) if (see(friend[k])) friend[k].X = friend[k].X.mul(m); /*immutable*/
        for (int k=0; k<nbOpponents; ++k) if (see(opponent[k])) opponent[k].X = opponent[k].X.mul(m); /*immutable*/
        for (int k=0; k<6; ++k) if (see(node[k])) node[k].X = node[k].X.mul(m); /*immutable*/
    }

    // setze die alten Knoten auf null
    for (int i=0; i<6; i++) node_[i] = null;

    // ueberschreibe mit eventuellen neuen Daten
    for (int i=0; i<pucks.length; ++i) {
    	if (pucks[i].X != null) // ich muss sie auch sehen, nicht nur hoeren
    	{
    		if (pucks[i].equals(BALL))
    		{ // ein Ball ist's
    			if (pucks[i].t==0) ball = new Puck(pucks[i]);
    			else if (pucks[i].t==self.t) goal = new Puck(pucks[i]); // es ist ein Tor
    			else flop = new Puck(pucks[i]); // es ist ein Flop
    		}
    		else if (pucks[i].equals(UNIT))
    		{ // eine Unit ist's
      			if (pucks[i].t==self.t) {
      				if (pucks[i].message.length()>0)
      					friend[Integer.parseInt(pucks[i].message)] = new Puck(pucks[i]); // ... noch dazu aus meinem Team
      			}
      			else
      			{ // ... vom Gegnerteam
      				boolean found = false;
      				for (int j=0; j<nbOpponents; ++j) if (opponent[j].id.equals(pucks[i].id))
      				{
      					opponent[j] = new Puck(pucks[i]);
      					found = true;
      				}
      				if ((!found) && (nbOpponents<3)) opponent[nbOpponents++] = new Puck(pucks[i]);
      			}
    		}
    		else if (pucks[i].equals(NODE)) // ein Knoten ist's
    		{
    			node[pucks[i].n-1] = new Puck(pucks[i]);
    			node_[pucks[i].n-1] = new Puck(pucks[i]);
    		}
    	}
    }
    // vielleicht ist pucksLocated jetzt true?
    if (!pucksLocated) pucksLocated = locate();
}

  boolean locate() {
  // der Spieler kann erst dann loslegen, wenn er gewisse Pucks lokalisiert hat
    if (ball==null) return false;
    if (goal==null) return false;
    for (int i=0; i<3; i++) if (friend[i]==null) return false;
    for (int i=0;i<6; i++) if (node[i]==null) return false;
    return true;
  }
}
